#!/bin/sh

vsize=192
case "$1" in
  -h|--help)
    echo "Usage: $0 [vsize]"
    echo ""
    echo "Default vsize is $vsize"
    exit
    ;;
  "") : ;;
  *) vsize=$1 ;;
esac

build() {
    gcc -E ../src/checker.c -DFILTER_MODE_$1 -o checker.c
    if [ "X$1" != "XNONE" ] ; then
        set x "$1" checker_bl.c
        ../tools/transform checker.c checker \
            --transform all \
            --change-indicator transformed \
            -o "$3"
    else
        set x "$1" checker.c
    fi
    ../tools/compile.sh checker checker.cl "$3"
}

render() {
    ../tools/renderer $1 \
        --fps \
        --perspective \
        --zoom 0 \
        --size 256x256 \
        -o $2 \
        --msaa $3
    case $2 in
      *.ppm)
        convert -set colorspace RGB "$2" \
            -crop 256x${vsize}+0+0 +repage \
            `basename $2 .ppm`.png
        rm "$2"
        ;;
    esac
}

build NONE
render checker.cl /dev/null 1
render checker.cl /dev/null 4
render checker.cl checker1-target.ppm 2048

build GAUSS
render checker.cl checker1-gauss.ppm 1

build RECT
render checker.cl checker1-box.ppm 1

build TENT
render checker.cl checker1-tent.ppm 1

rm checker.c checker.cl checker_bl.c
