#!/bin/sh

vsize=192
case "$1" in
  -h|--help)
    echo "Usage: $0 [vsize]"
    echo ""
    echo "Default vsize is $vsize"
    exit
    ;;
  "") : ;;
  *) vsize=$1 ;;
esac

build() {
    gcc -E ../src/checker2.c -DFILTER_MODE_$1 -o checker2.c
    if [ "X$1" != "XNONE" ] ; then
        set x "$1" checker2_bl.c
        ../tools/transform checker2.c checker2 \
            --transform all \
            --change-indicator transformed \
            -o "$3"
    else
        set x "$1" checker2.c
    fi
    ../tools/compile.sh checker2 checker2.cl "$3"
}

render() {
    ../tools/renderer checker2.cl \
        --fps \
        --perspective \
        --zoom 0 \
        --size 256x256 \
        -o $1 \
        --msaa $2 \
        ${3:+--diff l2 -g $3}
}

resize() {
    if [ $# -gt 1 ] ; then
        set false "$1" "$2"
    else
        set true "$1" `mktemp --suffix .ppm`
    fi
    convert "$2" -set colorspace RGB -crop 256x${vsize}+0+0 +repage "$3"
    if $1 ; then
        mv "$3" "$2"
    fi
}

inset() {
    convert -set colorspace RGB \
        $2 -resize 33% \
        $1 -reverse \
        -flip -flop -mosaic -flop -flip \
        $3
}

build NONE
render checker2-target.ppm 2048
render checker2-tmp.ppm 16
render checker2-diff.ppm 16 checker2-target.ppm
resize checker2-tmp.ppm
resize checker2-diff.ppm
inset checker2-tmp.ppm checker2-diff.ppm checker2-msaa.png

build TENT
render checker2-tmp.ppm 1
render checker2-diff.ppm 1 checker2-target.ppm
resize checker2-tmp.ppm
resize checker2-diff.ppm
inset checker2-tmp.ppm checker2-diff.ppm checker2-tent.png

resize checker2-target.ppm checker2-target.png

rm checker2-tmp.ppm checker2-diff.ppm checker2-target.ppm
rm checker2.c checker2_bl.c checker2.cl
