#!/bin/sh

vsize=192
case "$1" in
  -h|--help)
    echo "Usage: $0 [vsize]"
    echo ""
    echo "Default vsize is $vsize"
    exit
    ;;
  "") : ;;
  *) vsize=$1 ;;
esac

if [ ! -r circles-simplex.cl ] ; then
    echo "This script requires circles-simplex.cl to be linked in this directory"
    exit 1
fi

build() {
    gcc -E ../src/circles.c -DFILTER_MODE_$1 -o circles.c
    if [ "X$1" != "XNONE" ] ; then
        set x circles_bl.c "$2"
        ../tools/transform circles.c circles \
            ${3:+--genome} \
            ${3:---transform all} \
            --change-indicator transformed \
            -o "$2"
    else
        set x circles.c
    fi
    ../tools/compile.sh circles circles.cl "$2"
}

render() {
    ../tools/renderer $1 \
        --fps \
        --perspective \
        --zoom 0 \
        --size 256x256 \
        -o $2 \
        --msaa $3
    case $2 in
      *.ppm)
        convert -set colorspace RGB "$2" \
            -crop 256x${vsize}+0+0 +repage \
            `basename $2 .ppm`.png
        rm "$2"
        ;;
    esac
}

build NONE
render circles.cl /dev/null 1
render circles.cl /dev/null 16
render circles.cl circles-target.ppm 2048

build TENT
render circles.cl circles-tent.ppm 1

build TENT 00001
render circles.cl circles-genetic.ppm 1

render circles-simplex.cl circles-simplex.ppm 1

rm circles.c circles.cl circles_bl.c
