#!/bin/sh

shaders="
checker
gabor_brick
gabor_marble
gabor_rust
gabor_wood
gabor_harlequin
perlin
"

vsize=185
case "$1" in
  -h|--help)
    echo "Usage: $0 [vsize]"
    echo ""
    echo "Default vsize is $vsize"
    exit
    ;;
  "") : ;;
  *) vsize=$1 ;;
esac

missing=0
for shader in $shaders ; do
    if [ ! -r $shader-simplex.cl ] ; then
        missing=`expr $missing + 1`
        echo "missing $shader-simplex.cl"
    fi
done
if [ $missing -gt 0 ] ; then
    echo "missing $missing shaders. Exiting"
    exit 1
fi

build() {
    set x $1 `mktemp --suffix .c` `mktemp --suffix .cl`
    gcc -E ../src/$2.c -DFILTER_MODE_NONE -o $3
    ../tools/compile.sh $2 $4 $3
    rm $3
    echo $4
}

render() {
    ../tools/renderer $1 \
        --fps \
        --perspective \
        --zoom 0 \
        --size 256x256 \
        -o $2 \
        ${3:+--msaa $3}
    case $2 in
      *.ppm)
        convert -set colorspace RGB "$2" \
            -crop 256x${vsize}+0+0 +repage \
            `basename $2 .ppm`.png
        rm "$2"
        ;;
    esac
}

rows=0
cmd="montage -tile 1x$rows -mode Concatenate"
for shader in $shaders ; do
    rows=`expr $rows + 1`
    echo "===== $shader ====="
    cl=`build $shader`
    ../tools/montage.py \
        --persp 0 \
        --diff l2 \
        --crop 256x${vsize} \
        montage/ 256x256 $shader \
        golden $cl msaa $shader-simplex.cl
    cmd="$cmd montage/$shader.png"
    rm $cl
done

$cmd all.png

