#include <math.h>

#include "api3d.h"

/*********
 * float vector math functions
 *********/

float
abs_f( float a )
{
    return fabs( a );
}

float
ceil_f( float a )
{
    return ceil( a );
}

float
dot_f( float a, float b )
{
    return a * b;
}

float
floor_f( float a )
{
    return floor( a );
}

float
fract_f( float a )
{
    return a - floor( a );
}

float
length_f( float a )
{
    return a;
}

float
saturate_f( float a )
{
    return fmax( 0, fmin( 1, a ) );
}

float
smoothstep_f( float a, float b, float x )
{
    float t = saturate_f( (x - a) / (b -a) );
    return t * t * ( 3.0f - ( 2.0f * t ) );
}

float
step_f( float a, float b )
{
    return a <= b ? 1 : 0;
}

float2
makef2_f( float f )
{
    float2 result = { f, f };
    return result;
}

float2
makef2_ff( float x, float y )
{
    float2 result = { x, y };
    return result;
}

float2
float2_i2( int2 i )
{
    float2 result = { i.x, i.y };
    return result;
}

float2
add_f2f( float2 v, float s )
{
    float2 ans = { v.x + s, v.y + s };
    return ans;
}

float2
add_f2f2( float2 a, float2 b )
{
    float2 result = { a.x + b.x, a.y + b.y };
    return result;
}

float2
sub_ff2( float s, float2 v )
{
    float2 ans = { s - v.x, s - v.y };
    return ans;
}

float2
sub_f2f2( float2 a, float2 b )
{
    float2 ans = { a.x - b.x, a.y - b.y };
    return ans;
}

float2
mul_f2f( float2 v, float s )
{
    float2 ans = { v.x * s, v.y * s };
    return ans;
}

float2
mul_f2f2( float2 a, float2 b )
{
    float2 ans = { a.x * b.x, a.y * b.y };
    return ans;
}

float2
div_f2f( float2 v, float s )
{
    float2 ans = { v.x / s, v.y / s };
    return ans;
}

float2
div_f2f2( float2 a, float2 b )
{
    float2 ans = { a.x / b.x, a.y / b.y };
    return ans;
}

int2
ge_f2f2( float2 a, float2 b )
{
    int2 ans = { a.x >= b.x, a.y >= b.y };
    return ans;
}

int2
gt_f2f2( float2 a, float2 b )
{
    int2 ans = { a.x > b.x, a.y > b.y };
    return ans;
}

float2
abs_f2( float2 v )
{
    float2 result = { fabs( v.x ), fabs( v.y ) };
    return result;
}

float2
ceil_f2( float2 v )
{
    float2 result = { ceil( v.x ), ceil( v.y ) };
    return result;
}

float
dot_f2( float2 a, float2 b )
{
    return a.x * b.x + a.y * b.y;
}

float2
floor_f2( float2 v )
{
    float2 result = { floor( v.x ), floor( v.y ) };
    return result;
}

float2
fract_f2( float2 v )
{
    float2 result = {
        v.x - floor( v.x ),
        v.y - floor( v.y )
    };
    return result;
}

float
length_f2( float2 v )
{
    return sqrt( v.x * v.x + v.y * v.y );
}

float2
normalize_f2( float2 v )
{
    return div_f2f( v, length_f2( v ) );
}

float2
saturate_f2( float2 v )
{
    float2 result = {
        fmax( 0, fmin( 1, v.x ) ),
        fmax( 0, fmin( 1, v.y ) )
    };
    return result;
}

float2
smoothstep_f2( float2 a, float2 b, float2 x )
{
    float2 result = {
        smoothstep_f( a.x, b.x, x.x ),
        smoothstep_f( a.y, b.y, x.y )
    };
    return result;
}

float2
step_f2( float2 a, float2 b )
{
    float2 result = {
        a.x <= b.x ? 1 : 0,
        a.y <= b.y ? 1 : 0
    };
    return result;
}

float2
swizzle_f2xx( float2 v )
{
    float2 result = { v.x, v.x };
    return result;
}

float2
swizzle_f2yx( float2 v )
{
    float2 result = { v.y, v.x };
    return result;
}

float2
swizzle_f2yy( float2 v )
{
    float2 result = { v.y, v.y };
    return result;
}

float3
makef3_f( float v )
{
    float3 result = { v, v, v };
    return result;
}

float3
makef3_f2f( float2 xy, float z )
{
    float3 result = { xy.x, xy.y, z };
    return result;
}

float3
makef3_fff( float x, float y, float z )
{
    float3 result = { x, y, z };
    return result;
}

float3
float3_i3( int3 i )
{
    float3 result = { i.x, i.y, i.z };
    return result;
}

float3
add_f3f( float3 v, float s )
{
    float3 ans = { v.x + s, v.y + s, v.z + s };
    return ans;
}

float3
add_f3f3( float3 a, float3 b )
{
    float3 ans = { a.x + b.x, a.y + b.y, a.z + b.z };
    return ans;
}

float3
sub_ff3( float s, float3 v )
{
    float3 ans = { s - v.x, s - v.y, s - v.z };
    return ans;
}

float3
sub_f3f3( float3 a, float3 b )
{
    float3 ans = { a.x - b.x, a.y - b.y, a.z - b.z };
    return ans;
}

float3
mul_f3f( float3 v, float s )
{
    float3 ans = { v.x * s, v.y * s, v.z * s };
    return ans;
}

float3
mul_f3f3( float3 a, float3 b )
{
    float3 ans = { a.x * b.x, a.y * b.y, a.z * b.z };
    return ans;
}

float3
div_f3f( float3 v, float s )
{
    float3 ans = { v.x / s, v.y / s, v.z / s };
    return ans;
}

float3
div_f3f3( float3 a, float3 b )
{
    float3 ans = { a.x / b.x, a.y / b.y, a.z / b.z };
    return ans;
}

int3
ge_f3f3( float3 a, float3 b )
{
    int3 ans = { a.x >= b.x, a.y >= b.y, a.z >= b.z };
    return ans;
}

int3
gt_f3f3( float3 a, float3 b )
{
    int3 ans = { a.x > b.x, a.y > b.y, a.z > b.z };
    return ans;
}

float3
abs_f3( float3 xyz )
{
    float3 ans = { fabs( xyz.x ), fabs( xyz.y ), fabs( xyz.z ) };
    return ans;
}

float3
ceil_f3( float3 xyz )
{
    float3 result = { ceil( xyz.x ), ceil( xyz.y ), ceil( xyz.z ) };
    return result;
}

float3
cross( float3 u, float3 v )
{
    float3 result = {
        u.y * v.z - u.z * v.y,
        u.z * v.x - u.x * v.z,
        u.x * v.y - u.y * v.x
    };
    return result;
}

float
dot_f3( float3 a, float3 b )
{
    return a.x * b.x + a.y * b.y + a.z * b.z;
}

float3
floor_f3( float3 xyz )
{
    float3 result = { floor( xyz.x ), floor( xyz.y ), floor( xyz.z ) };
    return result;
}

float3
fract_f3( float3 v )
{
    float3 result = {
        v.x - floor( v.x ),
        v.y - floor( v.y ),
        v.z - floor( v.z )
    };
    return result;
}

float
length_f3( float3 v )
{
    return sqrt( v.x * v.x + v.y * v.y + v.z * v.z );
}

float3
normalize_f3( float3 v )
{
    return div_f3f( v, length_f3( v ) );
}

float3
saturate_f3( float3 xyz )
{
    float3 result = {
        fmax( 0, fmin( 1, xyz.x ) ),
        fmax( 0, fmin( 1, xyz.y ) ),
        fmax( 0, fmin( 1, xyz.z ) )
    };
    return result;
}

float3
smoothstep_f3( float3 a, float3 b, float3 x )
{
    float3 result = {
        smoothstep_f( a.x, b.x, x.x ),
        smoothstep_f( a.y, b.y, x.y ),
        smoothstep_f( a.z, b.z, x.z )
    };
    return result;
}

float3
step_f3( float3 a, float3 b )
{
    float3 result = {
        a.x <= b.x ? 1 : 0,
        a.y <= b.y ? 1 : 0,
        a.z <= b.z ? 1 : 0
    };
    return result;
}

float2
swizzle_f3xx( float3 v )
{
    float2 result = { v.x, v.x };
    return result;
}

float2
swizzle_f3xy( float3 v )
{
    float2 result = { v.x, v.y };
    return result;
}

float2
swizzle_f3xz( float3 v )
{
    float2 result = { v.x, v.z };
    return result;
}

float2
swizzle_f3yx( float3 v )
{
    float2 result = { v.y, v.x };
    return result;
}

float2
swizzle_f3yy( float3 v )
{
    float2 result = { v.y, v.y };
    return result;
}

float2
swizzle_f3yz( float3 v )
{
    float2 result = { v.y, v.z };
    return result;
}

float2
swizzle_f3zx( float3 v )
{
    float2 result = { v.z, v.x };
    return result;
}

float2
swizzle_f3zy( float3 v )
{
    float2 result = { v.z, v.y };
    return result;
}

float2
swizzle_f3zz( float3 v )
{
    float2 result = { v.z, v.z };
    return result;
}

float3
swizzle_f3xxx( float3 v )
{
    float3 result = { v.x, v.x, v.x };
    return result;
}

float3
swizzle_f3xxy( float3 v )
{
    float3 result = { v.x, v.x, v.y };
    return result;
}

float3
swizzle_f3xxz( float3 v )
{
    float3 result = { v.x, v.x, v.z };
    return result;
}

float3
swizzle_f3xyx( float3 v )
{
    float3 result = { v.x, v.y, v.x };
    return result;
}

float3
swizzle_f3xyy( float3 v )
{
    float3 result = { v.x, v.y, v.y };
    return result;
}

float3
swizzle_f3xzx( float3 v )
{
    float3 result = { v.x, v.z, v.x };
    return result;
}

float3
swizzle_f3xzy( float3 v )
{
    float3 result = { v.x, v.z, v.y };
    return result;
}

float3
swizzle_f3xzz( float3 v )
{
    float3 result = { v.x, v.z, v.z };
    return result;
}

float3
swizzle_f3yxx( float3 v )
{
    float3 result = { v.y, v.x, v.x };
    return result;
}

float3
swizzle_f3yxy( float3 v )
{
    float3 result = { v.y, v.x, v.y };
    return result;
}

float3
swizzle_f3yxz( float3 v )
{
    float3 result = { v.y, v.x, v.z };
    return result;
}

float3
swizzle_f3yyx( float3 v )
{
    float3 result = { v.y, v.y, v.x };
    return result;
}

float3
swizzle_f3yyy( float3 v )
{
    float3 result = { v.y, v.y, v.y };
    return result;
}

float3
swizzle_f3yyz( float3 v )
{
    float3 result = { v.y, v.y, v.z };
    return result;
}

float3
swizzle_f3yzx( float3 v )
{
    float3 result = { v.y, v.z, v.x };
    return result;
}

float3
swizzle_f3yzy( float3 v )
{
    float3 result = { v.y, v.z, v.y };
    return result;
}

float3
swizzle_f3yzz( float3 v )
{
    float3 result = { v.y, v.z, v.z };
    return result;
}

float3
swizzle_f3zxx( float3 v )
{
    float3 result = { v.z, v.x, v.x };
    return result;
}

float3
swizzle_f3zxy( float3 v )
{
    float3 result = { v.z, v.x, v.y };
    return result;
}

float3
swizzle_f3zxz( float3 v )
{
    float3 result = { v.z, v.x, v.z };
    return result;
}

float3
swizzle_f3zyx( float3 v )
{
    float3 result = { v.z, v.y, v.x };
    return result;
}

float3
swizzle_f3zyy( float3 v )
{
    float3 result = { v.z, v.y, v.y };
    return result;
}

float3
swizzle_f3zyz( float3 v )
{
    float3 result = { v.z, v.y, v.z };
    return result;
}

float3
swizzle_f3zzx( float3 v )
{
    float3 result = { v.z, v.z, v.x };
    return result;
}

float3
swizzle_f3zzy( float3 v )
{
    float3 result = { v.z, v.z, v.y };
    return result;
}

float3
swizzle_f3zzz( float3 v )
{
    float3 result = { v.z, v.z, v.z };
    return result;
}

float4
makef4_f( float v )
{
    float4 result = { v, v, v, v };
    return result;
}

float4
makef4_f3f( float3 xyz, float w )
{
    float4 result = { xyz.x, xyz.y, xyz.z, w };
    return result;
}

float4
makef4_ffff( float x, float y, float z, float w )
{
    float4 result = { x, y, z, w };
    return result;
}

float4
float4_i4( int4 i )
{
    float4 result = { i.x, i.y, i.z, i.w };
    return result;
}

float4
add_f4f( float4 v, float s )
{
    float4 ans = { v.x + s, v.y + s, v.z + s, v.w + s };
    return ans;
}

float4
add_f4f4( float4 a, float4 b )
{
    float4 ans = { a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w };
    return ans;
}

float4
sub_ff4( float s, float4 v )
{
    float4 ans = { s - v.x, s - v.y, s - v.z, s - v.w };
    return ans;
}

float4
sub_f4f4( float4 a, float4 b )
{
    float4 ans = { a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w };
    return ans;
}

float4
mul_f4f( float4 v, float s )
{
    float4 result = { v.x * s, v.y * s, v.z * s, v.w * s };
    return result;
}

float4
mul_f4f4( float4 a, float4 b )
{
    float4 result = { a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w };
    return result;
}

float4
div_f4f( float4 v, float s )
{
    float4 result = { v.x / s, v.y / s, v.z / s, v.w / s };
    return result;
}

float4
div_f4f4( float4 a, float4 b )
{
    float4 result = { a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w };
    return result;
}

int4
ge_f4f4( float4 a, float4 b )
{
    int4 result = { a.x >= b.x, a.y >= b.y, a.z >= b.z, a.w >= b.w };
    return result;
}

int4
gt_f4f4( float4 a, float4 b )
{
    int4 result = { a.x > b.x, a.y > b.y, a.z > b.z, a.w > b.w };
    return result;
}

float4
abs_f4( float4 v )
{
    float4 result = { fabs( v.x ), fabs( v.y ), fabs( v.z ), fabs( v.w ) };
    return result;
}

float4
ceil_f4( float4 v )
{
    float4 result = { ceil( v.x ), ceil( v.y ), ceil( v.z ), ceil( v.w ) };
    return result;
}

float
dot_f4( float4 a, float4 b )
{
    return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
}

float4
floor_f4( float4 v )
{
    float4 result = { floor( v.x ), floor( v.y ), floor( v.z ), floor( v.w ) };
    return result;
}

float4
fract_f4( float4 v )
{
    float4 result = {
        v.x - floor( v.x ),
        v.y - floor( v.y ),
        v.z - floor( v.z ),
        v.w - floor( v.w )
    };
    return result;
}

float
length_f4( float4 v )
{
    return sqrt( v.x * v.x + v.y * v.y + v.z * v.z + v.w * v.w );
}

float4
normalize_f4( float4 v )
{
    return div_f4f( v, length_f4( v ) );
}

float4
saturate_f4( float4 v )
{
    float4 result = {
        fmax( 0, fmin( 1, v.x ) ),
        fmax( 0, fmin( 1, v.y ) ),
        fmax( 0, fmin( 1, v.z ) ),
        fmax( 0, fmin( 1, v.w ) )
    };
    return result;
}

float4
smoothstep_f4( float4 a, float4 b, float4 x )
{
    float4 result = {
        smoothstep_f( a.x, b.x, x.x ),
        smoothstep_f( a.y, b.y, x.y ),
        smoothstep_f( a.z, b.z, x.z ),
        smoothstep_f( a.w, b.w, x.w )
    };
    return result;
}

float4
step_f4( float4 a, float4 b )
{
    float4 result = {
        a.x <= b.x ? 1 : 0,
        a.y <= b.y ? 1 : 0,
        a.z <= b.z ? 1 : 0,
        a.w <= b.w ? 1 : 0
    };
    return result;
}

float2
swizzle_f4xx( float4 v )
{
    float2 result = { v.x, v.x };
    return result;
}

float2
swizzle_f4xy( float4 v )
{
    float2 result = { v.x, v.y };
    return result;
}

float2
swizzle_f4xz( float4 v )
{
    float2 result = { v.x, v.z };
    return result;
}

float2
swizzle_f4xw( float4 v )
{
    float2 result = { v.x, v.w };
    return result;
}

float2
swizzle_f4yx( float4 v )
{
    float2 result = { v.y, v.x };
    return result;
}

float2
swizzle_f4yy( float4 v )
{
    float2 result = { v.y, v.y };
    return result;
}

float2
swizzle_f4yz( float4 v )
{
    float2 result = { v.y, v.z };
    return result;
}

float2
swizzle_f4yw( float4 v )
{
    float2 result = { v.y, v.w };
    return result;
}

float2
swizzle_f4zx( float4 v )
{
    float2 result = { v.z, v.x };
    return result;
}

float2
swizzle_f4zy( float4 v )
{
    float2 result = { v.z, v.y };
    return result;
}

float2
swizzle_f4zz( float4 v )
{
    float2 result = { v.z, v.z };
    return result;
}

float2
swizzle_f4zw( float4 v )
{
    float2 result = { v.z, v.w };
    return result;
}

float2
swizzle_f4wx( float4 v )
{
    float2 result = { v.w, v.x };
    return result;
}

float2
swizzle_f4wy( float4 v )
{
    float2 result = { v.w, v.y };
    return result;
}

float2
swizzle_f4wz( float4 v )
{
    float2 result = { v.w, v.z };
    return result;
}

float2
swizzle_f4ww( float4 v )
{
    float2 result = { v.w, v.w };
    return result;
}

float3
swizzle_f4xxx( float4 v )
{
    float3 result = { v.x, v.x, v.x };
    return result;
}

float3
swizzle_f4xyz( float4 v )
{
    float3 result = { v.x, v.y, v.z };
    return result;
}

float3
swizzle_f4yzw( float4 v )
{
    float3 result = { v.y, v.z, v.w };
    return result;
}

float4
swizzle_f4zyxw( float4 v )
{
    float4 result = { v.z, v.y, v.x, v.w };
    return result;
}

void
swizzleset_f4zw( float4 * v, float2 xy )
{
    v->z = xy.x;
    v->w = xy.y;
}

void
swizzleset_f4xyz( float4 * v, float3 xyz )
{
    v->x = xyz.x;
    v->y = xyz.y;
    v->z = xyz.z;
}

void
swizzleset_f4yzw( float4 * v, float3 xyz )
{
    v->y = xyz.x;
    v->z = xyz.y;
    v->w = xyz.z;
}

/*********
 * int vector math
 *********/

int2
makei2_i( int i )
{
    int2 result = { i, i };
    return result;
}

int2
makei2_ii( int x, int y )
{
    int2 result = { x, y };
    return result;
}

int2
int2_f2( float2 f  )
{
    int2 result = { f.x, f.y };
    return result;
}

int2
and_i2i( int2 v, int s )
{
    int2 ans = { v.x & s, v.y & s };
    return ans;
}

int2
and_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x & b.x, a.y & b.y };
    return ans;
}

int2
or_i2i( int2 v, int s )
{
    int2 ans = { v.x | s, v.y | s };
    return ans;
}

int2
or_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x | b.x, a.y | b.y };
    return ans;
}

int2
xor_i2i( int2 v, int s )
{
    int2 ans = { v.x ^ s, v.y ^ s };
    return ans;
}

int2
xor_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x ^ b.x, a.y ^ b.y };
    return ans;
}

int2
add_i2i( int2 v, int s )
{
    int2 ans = { v.x + s, v.y + s };
    return ans;
}

int2
add_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x + b.x, a.y + b.y };
    return ans;
}

int2
sub_ii2( int s, int2 v )
{
    int2 ans = { s - v.x, s - v.y };
    return ans;
}

int2
sub_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x - b.x, a.y - b.y };
    return ans;
}

int2
mul_i2i( int2 v, int s )
{
    int2 ans = { v.x * s, v.y * s };
    return ans;
}

int2
mul_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x * b.x, a.y * b.y };
    return ans;
}

int2
div_i2i( int2 v, int s )
{
    int2 ans = { v.x / s, v.y / s };
    return ans;
}

int2
div_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x / b.x, a.y / b.y };
    return ans;
}

int2
ge_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x >= b.x, a.y >= b.y };
    return ans;
}

int2
gt_i2i2( int2 a, int2 b )
{
    int2 ans = { a.x > b.x, a.y > b.y };
    return ans;
}

int2
swizzle_i2xx( int2 v )
{
    int2 result = { v.x, v.x };
    return result;
}

int2
swizzle_i2yx( int2 v )
{
    int2 result = { v.y, v.x };
    return result;
}

int2
swizzle_i2yy( int2 v )
{
    int2 result = { v.y, v.y };
    return result;
}

int3
makei3_i( int v )
{
    int3 result = { v, v, v };
    return result;
}

int3
makei3_i2i( int2 xy, int z )
{
    int3 result = { xy.x, xy.y, z };
    return result;
}

int3
makei3_iii( int x, int y, int z )
{
    int3 result = { x, y, z };
    return result;
}

int3
int3_f3( float3 f )
{
    int3 result = { f.x, f.y, f.z };
    return result;
}

int3
and_i3i( int3 v, int s )
{
    int3 result = { v.x & s, v.y & s, v.z & s };
    return result;
}

int3
and_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x & b.x, a.y & b.y, a.z & b.z };
    return ans;
}

int3
or_i3i( int3 v, int s )
{
    int3 ans = { v.x | s, v.y | s, v.z | s };
    return ans;
}

int3
or_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x | b.x, a.y | b.y, a.z | b.z };
    return ans;
}

int3
xor_i3i( int3 v, int s )
{
    int3 ans = { v.x ^ s, v.y ^ s, v.z ^ s };
    return ans;
}

int3
xor_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x ^ b.x, a.y ^ b.y, a.z ^ b.z };
    return ans;
}

int3
add_i3i( int3 v, int s )
{
    int3 ans = { v.x + s, v.y + s, v.z + s };
    return ans;
}

int3
add_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x + b.x, a.y + b.y, a.z + b.z };
    return ans;
}

int3
sub_ii3( int s, int3 v )
{
    int3 ans = { s - v.x, s - v.y, s - v.z };
    return ans;
}

int3
sub_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x - b.x, a.y - b.y, a.z - b.z };
    return ans;
}

int3
mul_i3i( int3 v, int s )
{
    int3 ans = { v.x * s, v.y * s, v.z * s };
    return ans;
}

int3
mul_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x * b.x, a.y * b.y, a.z * b.z };
    return ans;
}

int3
div_i3i( int3 v, int s )
{
    int3 ans = { v.x / s, v.y / s, v.z / s };
    return ans;
}

int3
div_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x / b.x, a.y / b.y, a.z / b.z };
    return ans;
}

int3
ge_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x >= b.x, a.y >= b.y, a.z >= b.z };
    return ans;
}

int3
gt_i3i3( int3 a, int3 b )
{
    int3 ans = { a.x > b.x, a.y > b.y, a.z > b.z };
    return ans;
}

int2
swizzle_i3xx( int3 v )
{
    int2 result = { v.x, v.x };
    return result;
}

int2
swizzle_i3xy( int3 v )
{
    int2 result = { v.x, v.y };
    return result;
}

int2
swizzle_i3xz( int3 v )
{
    int2 result = { v.x, v.z };
    return result;
}

int2
swizzle_i3yx( int3 v )
{
    int2 result = { v.y, v.x };
    return result;
}

int2
swizzle_i3yy( int3 v )
{
    int2 result = { v.y, v.y };
    return result;
}

int2
swizzle_i3yz( int3 v )
{
    int2 result = { v.y, v.z };
    return result;
}

int2
swizzle_i3zx( int3 v )
{
    int2 result = { v.z, v.x };
    return result;
}

int2
swizzle_i3zy( int3 v )
{
    int2 result = { v.z, v.y };
    return result;
}

int2
swizzle_i3zz( int3 v )
{
    int2 result = { v.z, v.z };
    return result;
}
