#ifndef API3D_H
#define API3D_H

#define __global __attribute__((opencl(global)))
#define __kernel __attribute__((opencl(kernel)))
#define __local  __attribute__((opencl(local)))

/*********
 * float vector types
 *********/

typedef struct float2 {
    float x;
    float y;
} float2;

typedef struct float3 {
    float x;
    float y;
    float z;
} float3;

typedef struct float4 {
    float x;
    float y;
    float z;
    float w;
} float4;

/*********
 * double vector types
 *********/

typedef struct double2 {
    double x;
    double y;
} double2;

typedef struct double3 {
    double x;
    double y;
    double z;
} double3;

typedef struct double4 {
    double x;
    double y;
    double z;
    double w;
} double4;

/*********
 * int vector types
 *********/

typedef struct int2 {
    int x;
    int y;
} int2;

typedef struct int3 {
    int x;
    int y;
    int z;
} int3;

typedef struct int4 {
    int x;
    int y;
    int z;
    int w;
} int4;

/*********
 * float vector math functions
 *********/

float abs_f( float a );
float ceil_f( float a );
float dot_f( float a, float b );
float floor_f( float a );
float fract_f( float a );
float length_f( float a );
float lerp_ff( float a, float b, float s ) __attribute__((opencl(keep)));
float saturate_f( float a );
float smoothstep_f( float a, float b, float x );
float smoothpulse_f( float e0, float e1, float e2, float e3, float x ) __attribute__((opencl(keep)));
float sq_f( float a );
float step_f( float a, float b );
float trunc_f( float a ) __attribute__((opencl(keep)));

float2 makef2_f( float f );
float2 makef2_ff( float x, float y );
float2 float2_i2( int2 i );

float2 add_f2f( float2 v, float s );
float2 add_f2f2( float2 a, float2 b );
float2 sub_ff2( float s, float2 v );
float2 sub_f2f2( float2 a, float2 b );
float2 mul_f2f( float2 v, float s );
float2 mul_f2f2( float2 a, float2 b );
float2 div_f2f( float2 v, float s );
float2 div_f2f2( float2 a, float2 b );
int2 ge_f2f2( float2 a, float2 b );
int2 gt_f2f2( float2 a, float2 b );

float2 abs_f2( float2 v );
float2 ceil_f2( float2 v );
float dot_f2( float2 a, float2 b );
float2 floor_f2( float2 v );
float2 fract_f2( float2 v );
float length_f2( float2 v );
float2 lerp_f2f( float2 a, float2 b, float s ) __attribute__((opencl(keep)));
float2 normalize_f2( float2 v );
float2 saturate_f2( float2 v );
float2 smoothpulse_f2( float2 e0, float2 e1, float2 e2, float2 e3, float2 x ) __attribute__((opencl(keep)));
float2 smoothstep_f2( float2 a, float2 b, float2 x );
float2 sq_f2( float2 a );
float2 step_f2( float2 a, float2 b );
float2 trunc_f2( float2 a ) __attribute__((opencl(keep)));

float2 swizzle_f2xx( float2 v );
float2 swizzle_f2yx( float2 v );
float2 swizzle_f2yy( float2 v );

float3 makef3_f( float v );
float3 makef3_f2f( float2 xy, float z );
float3 makef3_fff( float x, float y, float z );
float3 float3_i3( int3 i );

float3 add_f3f( float3 v, float s );
float3 add_f3f3( float3 a, float3 b );
float3 sub_ff3( float s, float3 v );
float3 sub_f3f3( float3 a, float3 b );
float3 mul_f3f( float3 v, float s );
float3 mul_f3f3( float3 a, float3 b );
float3 div_f3f( float3 v, float s );
float3 div_f3f3( float3 a, float3 b );
int3 ge_f3f3( float3 a, float3 b );
int3 gt_f3f3( float3 a, float3 b );

float3 abs_f3( float3 xyz );
float3 ceil_f3( float3 xyz );
float dot_f3( float3 a, float3 b );
float3 cross( float3 u, float3 v );
float3 floor_f3( float3 xyz );
float3 fract_f3( float3 v );
float length_f3( float3 v );
float3 lerp_f3f( float3 a, float3 b, float s ) __attribute__((opencl(keep)));
float3 normalize_f3( float3 v );
float3 saturate_f3( float3 xyz );
float3 smoothpulse_f3( float3 e0, float3 e1, float3 e2, float3 e3, float3 x ) __attribute__((opencl(keep)));
float3 smoothstep_f3( float3 a, float3 b, float3 x );
float3 sq_f3( float3 a );
float3 step_f3( float3 a, float3 b );
float3 trunc_f3( float3 a ) __attribute__((opencl(keep)));

float2 swizzle_f3xx( float3 v );
float2 swizzle_f3xy( float3 v );
float2 swizzle_f3xz( float3 v );
float2 swizzle_f3yx( float3 v );
float2 swizzle_f3yy( float3 v );
float2 swizzle_f3yz( float3 v );
float2 swizzle_f3zx( float3 v );
float2 swizzle_f3zy( float3 v );
float2 swizzle_f3zz( float3 v );
float3 swizzle_f3xxx( float3 v );
float3 swizzle_f3xxy( float3 v );
float3 swizzle_f3xxz( float3 v );
float3 swizzle_f3xyx( float3 v );
float3 swizzle_f3xyy( float3 v );
float3 swizzle_f3xzx( float3 v );
float3 swizzle_f3xzy( float3 v );
float3 swizzle_f3xzz( float3 v );
float3 swizzle_f3yxx( float3 v );
float3 swizzle_f3yxy( float3 v );
float3 swizzle_f3yxz( float3 v );
float3 swizzle_f3yyx( float3 v );
float3 swizzle_f3yyy( float3 v );
float3 swizzle_f3yyz( float3 v );
float3 swizzle_f3yzx( float3 v );
float3 swizzle_f3yzy( float3 v );
float3 swizzle_f3yzz( float3 v );
float3 swizzle_f3zxx( float3 v );
float3 swizzle_f3zxy( float3 v );
float3 swizzle_f3zxz( float3 v );
float3 swizzle_f3zyx( float3 v );
float3 swizzle_f3zyy( float3 v );
float3 swizzle_f3zyz( float3 v );
float3 swizzle_f3zzx( float3 v );
float3 swizzle_f3zzy( float3 v );
float3 swizzle_f3zzz( float3 v );

float4 makef4_f( float v );
float4 makef4_f3f( float3 xyz, float w );
float4 makef4_ffff( float x, float y, float z, float w );
float4 float4_i4( int4 i );

float4 add_f4f( float4 v, float s );
float4 add_f4f4( float4 a, float4 b );
float4 sub_ff4( float s, float4 v );
float4 sub_f4f4( float4 a, float4 b );
float4 mul_f4f( float4 v, float s );
float4 mul_f4f4( float4 a, float4 b );
float4 div_f4f( float4 v, float s );
float4 div_f4f4( float4 a, float4 b );
int4 ge_f4f4( float4 a, float4 b );
int4 gt_f4f4( float4 a, float4 b );

float4 abs_f4( float4 v );
float4 ceil_f4( float4 v );
float dot_f4( float4 a, float4 b );
float4 floor_f4( float4 v );
float4 fract_f4( float4 v );
float length_f4( float4 v );
float4 lerp_f4f( float4 a, float4 b, float s ) __attribute__((opencl(keep)));
float4 normalize_f4( float4 v );
float4 saturate_f4( float4 v );
float4 smoothpulse_f4( float4 e0, float4 e1, float4 e2, float4 e3, float4 x ) __attribute__((opencl(keep)));
float4 smoothstep_f4( float4 a, float4 b, float4 x );
float4 sq_f4( float4 a );
float4 step_f4( float4 a, float4 b );
float4 trunc_f4( float4 a ) __attribute__((opencl(keep)));

float2 swizzle_f4xx( float4 v );
float2 swizzle_f4xy( float4 v );
float2 swizzle_f4xz( float4 v );
float2 swizzle_f4xw( float4 v );
float2 swizzle_f4yx( float4 v );
float2 swizzle_f4yy( float4 v );
float2 swizzle_f4yz( float4 v );
float2 swizzle_f4yw( float4 v );
float2 swizzle_f4zx( float4 v );
float2 swizzle_f4zy( float4 v );
float2 swizzle_f4zz( float4 v );
float2 swizzle_f4zw( float4 v );
float2 swizzle_f4wx( float4 v );
float2 swizzle_f4wy( float4 v );
float2 swizzle_f4wz( float4 v );
float2 swizzle_f4ww( float4 v );
float3 swizzle_f4xxx( float4 v );
float3 swizzle_f4xyz( float4 v );
float3 swizzle_f4yzw( float4 v );
float4 swizzle_f4zyxw( float4 v );
void swizzleset_f4zw( float4 * v, float2 xy );
void swizzleset_f4xyz( float4 * v, float3 xyz );
void swizzleset_f4yzw( float4 * v, float3 xyz );

/*********
 * int vector math
 *********/

int2 makei2_i( int i );
int2 makei2_ii( int x, int y );
int2 int2_f2( float2 f );

int2 and_i2i( int2 v, int s );
int2 and_i2i2( int2 a, int2 b );
int2 or_i2i( int2 v, int s );
int2 or_i2i2( int2 a, int2 b );
int2 xor_i2i( int2 v, int s );
int2 xor_i2i2( int2 a, int2 b );
int2 add_i2i( int2 v, int s );
int2 add_i2i2( int2 a, int2 b );
int2 sub_ii2( int s, int2 v );
int2 sub_i2i2( int2 a, int2 b );
int2 mul_i2i( int2 v, int s );
int2 mul_i2i2( int2 a, int2 b );
int2 div_i2i( int2 v, int s );
int2 div_i2i2( int2 a, int2 b );
int2 ge_i2i2( int2 a, int2 b );
int2 gt_i2i2( int2 a, int2 b );

int2 swizzle_i2xx( int2 v );
int2 swizzle_i2yx( int2 v );
int2 swizzle_i2yy( int2 v );

int3 makei3_i( int v );
int3 makei3_i2i( int2 xy, int z );
int3 makei3_iii( int x, int y, int z );
int3 int3_f3( float3 f );

int3 and_i3i( int3 v, int s );
int3 and_i3i3( int3 a, int3 b );
int3 or_i3i( int3 v, int s );
int3 or_i3i3( int3 a, int3 b );
int3 xor_i3i( int3 v, int s );
int3 xor_i3i3( int3 a, int3 b );
int3 add_i3i( int3 v, int s );
int3 add_i3i3( int3 a, int3 b );
int3 sub_ii3( int s, int3 v );
int3 sub_i3i3( int3 a, int3 b );
int3 mul_i3i( int3 v, int s );
int3 mul_i3i3( int3 a, int3 b );
int3 div_i3i( int3 v, int s );
int3 div_i3i3( int3 a, int3 b );
int3 ge_i3i3( int3 a, int3 b );
int3 gt_i3i3( int3 a, int3 b );

int2 swizzle_i3xx( int3 v );
int2 swizzle_i3xy( int3 v );
int2 swizzle_i3xz( int3 v );
int2 swizzle_i3yx( int3 v );
int2 swizzle_i3yy( int3 v );
int2 swizzle_i3yz( int3 v );
int2 swizzle_i3zx( int3 v );
int2 swizzle_i3zy( int3 v );
int2 swizzle_i3zz( int3 v );

/*********
 * implementation of retained functions
 *********/

inline
float
lerp_ff( float a, float b, float s )
{
    return a + s * ( b - a );
}

inline
float
smoothpulse_f( float e0, float e1, float e2, float e3, float x )
{
    return smoothstep_f( e0, e1, x ), smoothstep_f( e2, e3, x );
}

inline
float
trunc_f( float a )
{
    return floor_f( a ) - step_f( 0.0f, a ) + 1;
}

inline
float2
lerp_f2f( float2 a, float2 b, float s )
{
    return add_f2f2( a, mul_f2f( sub_f2f2( b, a ), s ) );
}

inline
float2
smoothpulse_f2( float2 e0, float2 e1, float2 e2, float2 e3, float2 x )
{
    return sub_f2f2( smoothstep_f2( e0, e1, x ), smoothstep_f2( e2, e3, x ) );
}

inline
float2
trunc_f2( float2 a )
{
    return add_f2f( sub_f2f2( floor_f2( a ), step_f2( makef2_f( 0.0f ), a ) ), 1.0f );
}

inline
float3
lerp_f3f( float3 a, float3 b, float s )
{
    return add_f3f3( a, mul_f3f( sub_f3f3( b, a ), s ) );
}

inline
float3
smoothpulse_f3( float3 e0, float3 e1, float3 e2, float3 e3, float3 x )
{
    return sub_f3f3( smoothstep_f3( e0, e1, x ), smoothstep_f3( e2, e3, x ) );
}

inline
float3
trunc_f3( float3 a )
{
    return add_f3f( sub_f3f3( floor_f3( a ), step_f3( makef3_f( 0.0f ), a ) ), 1.0f );
}

inline
float4
lerp_f4f( float4 a, float4 b, float s )
{
    return add_f4f4( a, mul_f4f( sub_f4f4( b, a ), s ) );
}

inline
float4
smoothpulse_f4( float4 e0, float4 e1, float4 e2, float4 e3, float4 x )
{
    return sub_f4f4( smoothstep_f4( e0, e1, x ), smoothstep_f4( e2, e3, x ) );
}

inline
float4
trunc_f4( float4 a )
{
    return add_f4f( sub_f4f4( floor_f4( a ), step_f4( makef4_f( 0.0f ), a ) ), 1.0f );
}

#endif /* API3D_H */

