#include <math.h>

#include <stdio.h>

#include "api3d.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"

void
checker_aa(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float4 * result
) {
    float4 checkcolor;
    float swidth, twidth;
    float sfuzz, tfuzz, fuzzmax;
    float smod, tmod, x, y;

    /* Determine how wide in s-t space one pixel projects to */
    swidth = abs_f( in->dwdx.x );
    twidth = abs_f( in->dwdy.y );

    /* Figure out amount of fuzziness */
    sfuzz = 0.5 * swidth * 10;
    tfuzz = 0.5 * twidth * 10;
    fuzzmax = fmax( sfuzz, tfuzz );

    /* Get the place in the pattern where we're sampling */
    smod = fmod( fabs( in->objPos.x ) * 10 + 0.25, 1 );
    tmod = fmod( fabs( in->objPos.y ) * 10 + 0.25, 1 );

    /* If the filter width is small enough, compute the pattern color */
    if ( fuzzmax <= 0.5 ) {
        x = smoothstep_f( 0.5, 0.5 + sfuzz, smod ) + ( 1 - smoothstep_f( 0, sfuzz, smod ) );
        y = smoothstep_f( 0.5, 0.5 + tfuzz, tmod ) + ( 1 - smoothstep_f( 0, tfuzz, tmod ) );
        checkcolor = lerp_f4f( makef4_f( 1 ), makef4_f( 0 ), x * y + (1-x) * (1-y) );
        /* Gradually fade in the average color when we get close to the limit */
        *result = lerp_f4f( checkcolor, makef4_f( 0.5 ), smoothstep_f( 0.125, 0.5, fuzzmax ) );
    } else {
        /* Otherwise, only use the average color */
        *result = makef4_f( 0.5 );
    }
}
