#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"

#include "api3d_bl.c"

/* http://glslsandbox.com/e#24336.0 */

float
mylength( float2 pos )
{
    return abs_f( pos.x ) + abs_f( pos.y );
}

float smoothpulse(float a, float b, float x, float k) {
    float t1 = saturate_f((x - a)/k);
    float t2 = saturate_f((x - b + k)/k);
    return t1*t1*(3.0f - (2.0f*t1)) - t2*t2*(3.0f - (2.0f*t2));
}

float ring(float2 p, float r, float w) {
        float col = 0.0;
        float d = mylength(p);
        col = smoothpulse(r, r + w, d, 0.001);
        return col;
}


float hash(float px, float py) {
        return (sin(px * 15.75 + py * 35.89) * 43758.71);
}

float mymod(float x, float m) {
    return m * fract_f(abs_f(x) / m);
}

float4
crazymaze(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c) 
{
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );
        
    float time = 1.0f;
    float color = 0.0f;
    float dir = 0.0f;
    float rand; 
    float theta; 
    float2 p = makef2_ff(abs_f(in->objPos.x) * 10.0f, 
                         abs_f(in->objPos.y) * 10.0f) ; 
    float2 q = p; 

    p.x = mymod(p.x, 1.0f / 3.0f) - 1.0f / 3.0f * 0.5f; 
    p.y = mymod(p.y, 1.0f / 3.0f) - 1.0f / 3.0f * 0.5f; 
    
    rand = hash( floor_f(q.x * 3.0f) + 10.0f * time ,
                 floor_f(q.y * 3.0f) + 10.0f * time ) ; 
    dir = (1.0f - step_f(rand, 0.5f)) * rand;
    theta = M_PI * dir;

    // rotate by theta 
    q.x = cos(theta) * p.x - sin(theta) * p.y ; 
    q.y = sin(theta) * p.x + cos(theta) * p.y ; 

    color += ring(sub_f2f2(q , makef2_ff(0.17f,0.17f)), 0.155f, 0.03f);
    color += ring(add_f2f2(q , makef2_ff(0.17f,0.17f)), 0.155f, 0.03f);
    return makef4_ffff( color, color, color, 1.0f ) ; 
}
