#ifndef FACTORS_H
#define FACTORS_H

/*
 * We want a Gaussian in the spatial domain such that the standard deviation of
 * the Fourier transform of that Gaussian is r * 1/w where w is the sampling
 * rate.
 *
 * The Fourier transform of a normalized Gaussian with standard deviation c is
 *    exp(-2 * c*c * pi*pi * k*k)
 * which itself has a standard deviation of (1 / (2 * c * pi)).
 *
 * Thus, we can solve for c in terms of r and w:
 *    r / w = 1 / (2 * c * pi)
 *        c = w / (2 * r * pi)
 */
const float GAUSS_SIGMA_FACTOR = 2.0f / M_PI;

#endif /* FACTORS_H */

