#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"

#include "api3d_bl.c"

/* http://docs.unity3d.com/Manual/SL-VertexFragmentShaderExamples.html */

float4
fractal(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c) 
{
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );

    float4 color; 
    float2 mcoord; 
    float2 coord = makef2_ff(0.0,0.0);
    mcoord.x = ((1.0-in->objPos.x)*3.5)-2.5;
    mcoord.y = (in->objPos.y*2.0)-1.0;
    float iteration = 0.0;
    const float _MaxIter = 29.0;
    const float PI = 3.14159;
    float xtemp;
    for ( iteration = 0.0; iteration < _MaxIter; iteration += 1.0) {
      if ( coord.x*coord.x + coord.y*coord.y > 2.0*(cos(fmod(0.0,2.0*PI))+1.0) )
        break;
      xtemp = coord.x*coord.x - coord.y*coord.y + mcoord.x;
      coord.y = 2.0*coord.x*coord.y + mcoord.y;
      coord.x = xtemp;
    }
    float val = fmod((iteration/_MaxIter)+0.0,1.0);
    color = makef4_ffff(
                    (3.0*fabs(fmod(2.0*val,1.0)-0.5)),
                    (3.0*fabs(fmod(2.0*val+(1.0/3.0),1.0)-0.5)),
                    (3.0*fabs(fmod(2.0*val-(1.0/3.0),1.0)-0.5)),
              1.0
        ) ;

    return color;
}
