#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_helpers.h"
#include "procedural_shaders.h"
#include "reprojection.h"

/* Frame order:
 *   FILTER_MODE_fBm
 *   FILTER_MODE_NONE
 */
#include "gabor.h"

#include "api3d_bl.c"

const float radius_inner = 0.35;
const float radius_outer = 0.38;

float4
get_noise_f4( __global global_buffers * g, float2 p ) {
    return samplef4_ff(
        g->g_txRand_f4, g->sizex, g->sizey, p.x, p.y
    );
}

void
inner_loop(
    __global global_buffers * g,
    int i, int j, int k,
    float3 cell, float3 XYZfrac, float * priority, float4 * color
) {
    float3 cell_t, offset_t, offset_t3;
    float2 randomUV;
    float4 random, randcolor;
    float radius2;
    float cond1, cond2, cond3;

    cell_t = add_f3f3( cell, makef3_fff( i, j, k ) );
    offset_t = sub_f3f3( XYZfrac, makef3_fff( i, j, k ) );
    randomUV = add_f2f( mul_f2f2( swizzle_f3xy( cell_t ), makef2_ff( 0.037f, 0.119f ) ), cell_t.x * 0.013f );
    random = get_noise_f4( g, randomUV );
    offset_t3 = sub_f3f3( offset_t, add_f3f( swizzle_f4xyz( random ), 0.5f ) );
    radius2 = dot_f3( offset_t3, offset_t3 );
    randcolor = get_noise_f4( g, add_f2f2( randomUV, makef2_ff( 0.13f, 0.4f ) ) );

    cond1 = ( 1.0f - step_f( random.w, *priority ) );
    cond2 = ( 1.0f - step_f( radius_inner, radius2 ) );
    cond3 = ( 1.0f - step_f( radius_outer, radius2 ) );
    *color = add_f4f4(
        mul_f4f( randcolor, cond1 * cond2 ),
        mul_f4f( add_f4f4(
            mul_f4f( swizzle_f4zyxw( randcolor ), cond1 * cond3 ),
            mul_f4f( *color, 1.0f - cond1 * cond3 )
        ), 1.0f - cond1 * cond2 )
    );
    *priority =
        random.w * cond1 * cond2 +
        (
            random.w * cond1 * cond3 +
            *priority * ( 1.0f - cond1 * cond3 )
        ) * ( 1.0f - cond1 * cond2 );

    randomUV = add_f2f2( randomUV, makef2_ff( 0.077f, 0.219f ) );
    random = get_noise_f4( g, randomUV );
    offset_t3 = sub_f3f3( offset_t, add_f3f( swizzle_f4xyz( random ), 0.5f ) );
    radius2 = dot_f3( offset_t3, offset_t3 );
    randcolor = get_noise_f4( g, add_f2f2( randomUV, makef2_ff( 0.13f, 0.4f ) ) );
    cond1 = ( 1.0f - step_f( random.w, *priority ) );
    cond2 = ( 1.0f - step_f( radius_inner, radius2 ) );
    cond3 = ( 1.0f - step_f( radius_outer, radius2 ) );
    *color = add_f4f4(
        mul_f4f( randcolor, cond1 * cond2 ),
        mul_f4f( add_f4f4(
            mul_f4f( swizzle_f4zyxw( randcolor ), cond1 * cond3 ),
            mul_f4f( *color, 1.0f - cond1 * cond3 )
        ), 1.0f - cond1 * cond2 )
    );
    *priority =
        random.w * cond1 * cond2 +
        (
            random.w * cond1 * cond3 +
            *priority * ( 1.0f - cond1 * cond3 )
        ) * ( 1.0f - cond1 * cond2 );
}

float4
gabor_harlequin(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c
) {
    float3 scaledXYZ, XYZfrac;

    float3 cell;
    float4 color;
    float priority;
    int i, j, k;

    in->worldPos = shiftValue_f3( in, in->worldPos, offset );
    in->objPos = shiftValue_f3( in, in->objPos, offset );

    scaledXYZ = mul_f3f( in->objPos, 5 );
    cell = floor_f3( scaledXYZ );
    XYZfrac = fract_f3( scaledXYZ );

    color = makef4_ffff( 0xF3/255.0f, 0x93/255.0f, 0x22/255.0f, 0 );

    priority = -1;
    inner_loop( g, -1, -1, -1, cell, XYZfrac, &priority, &color );
    inner_loop( g, -1, -1,  0, cell, XYZfrac, &priority, &color );
    inner_loop( g, -1,  0, -1, cell, XYZfrac, &priority, &color );
    inner_loop( g, -1,  0,  0, cell, XYZfrac, &priority, &color );
    inner_loop( g,  0, -1, -1, cell, XYZfrac, &priority, &color );
    inner_loop( g,  0, -1,  0, cell, XYZfrac, &priority, &color );
    inner_loop( g,  0,  0, -1, cell, XYZfrac, &priority, &color );
    inner_loop( g,  0,  0,  0, cell, XYZfrac, &priority, &color );

    color = mul_f4f( color, 0.7f );
    return color;
}

