#include <math.h>
#include <stdio.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_helpers.h"
#include "procedural_shaders.h"
#include "reprojection.h"

/* Frame order:
 *   FILTER_MODE_fBm
 *   FILTER_MODE_NONE
 */
#include "gabor.h"

#include "api3d_bl.c"

#define PALE_BLUE   makef4_ffff( 1.00, 1.00, 1.00, 1 )
#define MEDIUM_BLUE makef4_ffff( 0.50, 0.50, 0.50, 1 )
#define DARK_BLUE   makef4_ffff( 0.40, 0.20, 0.20, 1 )
#define DARKER_BLUE makef4_ffff( 0.10, 0.10, 0.10, 1 )

/*
const float4 colorTable[13] = {
    PALE_BLUE, PALE_BLUE,
    MEDIUM_BLUE, MEDIUM_BLUE, MEDIUM_BLUE,
    PALE_BLUE, PALE_BLUE,
    DARK_BLUE, DARK_BLUE,
    DARKER_BLUE, DARKER_BLUE,
    PALE_BLUE, DARKER_BLUE
};
*/
const unsigned int nColorTableSize = 13;

float3
marbleColor( float m )
{
    float4 color;
    float nColorTableSizeM1, tentRadius, perc;
    unsigned int i;

const float4 colorTable01 = PALE_BLUE;
const float4 colorTable02 = PALE_BLUE;
const float4 colorTable03 = MEDIUM_BLUE;
const float4 colorTable04 = MEDIUM_BLUE;
const float4 colorTable05 = MEDIUM_BLUE;
const float4 colorTable06 = PALE_BLUE;
const float4 colorTable07 = PALE_BLUE;
const float4 colorTable08 = DARK_BLUE;
const float4 colorTable09 = DARK_BLUE;
const float4 colorTable10 = DARKER_BLUE;
const float4 colorTable11 = DARKER_BLUE;
const float4 colorTable12 = PALE_BLUE;
const float4 colorTable13 = PALE_BLUE;

    color = makef4_f( 0 );
    nColorTableSizeM1 = nColorTableSize - 1;
    tentRadius = 1 / nColorTableSizeM1;
    /*
    for ( i = 0; i < nColorTableSize; ++i ) {
        perc = (float) i / nColorTableSizeM1;
        color = add_f4f4( color, mul_f4f( colorTable[ i ], saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    }
    */
    perc = 0.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable01, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 1.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable02, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 2.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable03, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 3.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable04, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 4.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable05, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 5.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable06, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 6.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable07, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 7.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable08, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 8.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable09, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 9.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable10, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 10.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable11, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 11.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable12, saturate_f( tentRadius - abs_f( perc - m ) ) ) );
    perc = 12.0f / 12.0f;
    color = add_f4f4( color, mul_f4f( colorTable13, saturate_f( tentRadius - abs_f( perc - m ) ) ) );

    color = div_f4f( color, color.w );
    return swizzle_f4xyz( color );
}

float4
gabor_marble(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c
) {
    float3 colorOut;
    float3 centerPos;
    float radialNoise;

    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );

#ifndef FILTER_MODE_FBM
    float K = 1.0f;
    float a = 0.05f;
    float F_0 = 2.0f;
    float omega_0 = 0.0f;
    float impulses = 64.0f;
    unsigned period = 256;
#endif

    in->objPos  = shiftValue_f3( in, in->objPos, offset );

    centerPos = mul_f3f( in->objPos, 5.15f );

#ifdef FILTER_MODE_FBM
    radialNoise = fBm( g, makef4_f3f( centerPos, 0 ), 3, 2, 0.5, NT_SIMPLEX3D );
#else
    radialNoise = gabor_noise(
        K, a, F_0, omega_0,
        impulses, period,
        centerPos.x, centerPos.y
    );
#endif
    radialNoise = saturate_f( radialNoise + 0.36f );

    colorOut = marbleColor( radialNoise );

    return makef4_f3f( colorOut, 1 );
}

