#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"

/* Frame order:
 *   FILTER_MODE_fBm
 *   FILTER_MODE_NONE
 */

#include "gabor.h"

#include "api3d_bl.c"

float4
gabor_rust(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c
) {
    float4 colorMetal, colorRusted;
    float3 centerPos;
    float radialNoise, rustNoise;

    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );

#ifndef FILTER_MODE_FBM
    float K = 1.0f;
    float a = 0.05f;
    float F_0 = 1.0f;
    float omega_0 = 0.0f;
    float impulses = 32.0f;
    unsigned period = 256;
#endif

    in->objPos = shiftValue_f3( in, in->objPos, offset );

    centerPos = mul_f3f( in->objPos, 5.15f );

#ifdef FILTER_MODE_FBM
    radialNoise = fBm( g, makef4_f3f( centerPos, 0 ), 3, 2, 0.7, NT_SIMPLEX3D );
#else
    radialNoise = gabor_noise(
        K, a, F_0, omega_0,
        impulses, period,
        centerPos.x, centerPos.y
    );
#endif
    radialNoise = saturate_f( radialNoise + 0.34f );

    centerPos = mul_f3f( centerPos, 100.0f * 0.5f );

#ifdef FILTER_MODE_FBM
    rustNoise = fBm( g, makef4_f3f( centerPos, 0 ), 1, 2, 0.7, NT_SIMPLEX3D_TURB ) * 0.02;
#else
    rustNoise = gabor_noise(
        K, a, F_0, omega_0,
        impulses / 8, period,
        centerPos.x, centerPos.y
    ) * 0.02;
#endif

    colorMetal  = makef4_ffff( 0.5f, 0.50f, 0.50f, 1.0f );
    colorRusted = add_f4f( makef4_ffff( 0.2f, 0.06f, 0.04f, 1.0f ), -rustNoise );
    return lerp_f4f( colorRusted, colorMetal, radialNoise );
}

