#ifndef LIGHTING_H
#define LIGHTING_H

#include "api3d.h"
#include "procedural_shaders.h"

float4
lightingFunc(
    VS_OUTPUT * in,
    float3 vNormal,
    float4 surfaceColor,
    float specularExp /* = 128 */,
    float specularAmount /* = 1.0 */,
    int bClamp /* = true */
) {
    float3 vLightDir, vReflectionWS;
    float4 cSpecular, color;
    float fRdotL, lighting;

    vLightDir = normalize_f3( in->vLightDirVec );
    cSpecular = makef4_f( 0 );
    vReflectionWS = normalize_f3( sub_f3f3( mul_f3f( vNormal, 2 * dot_f3( in->vViewWS, vNormal ) ), in->vViewWS ) );
    fRdotL = saturate_f3( dot_f3( vReflectionWS, normalize_f3( vLightDir ) ) );
    cSpecular = makef4_f( saturate_f( pow( fRdotL, specularExp ) ) );
    cSpecular = specularExp == 0 ? 0 : cSpecular * specularAmount;

    lighting = 0.1 + 0.9 * saturate_f( dot_f3( vNormal, vLightDir ) );
    color = add_f4f4( mul_f4f( surfaceColor, lighting ), cSpecular );
    if ( bClamp )
        return saturate_f4( color );
    else
        return color;
}

#endif /* LIGHTING_H */

