#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"

#include "api3d_bl.c"

/* http://glslsandbox.com/e#24339.0 */ 

float
mylength( float2 pos )
{
    return abs_f( pos.x ) + abs_f( pos.y );
}

float
mydistance( float2 v1, float2 v2) 
{
  return mylength(sub_f2f2(v1,v2)); 
} 

float4
lightweave(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c) 
{
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );
        
    float time = 10.0 * abs_f(in->objPos.x + 2*in->objPos.y) ; 
    float size = 30.0;
    float color = 0.0;
    float resolution_xy = 2.0 ; 
    float2 inxy = makef2_ff(in->objPos.x * 500, in->objPos.y * 500) ; 

    float3 mColor = makef3_fff( 
        (sin(time * 0.1)+1.0)*0.5, 
        (sin(in->objPos.x * 0.1)+1.0)*0.5, 
        (sin(in->objPos.y * 0.1)+1.0)*0.5
        );

        float r = 100.0;
        float speed = 5.0;
        float mx = 0.5; 
        float my = 0.5;
        float2 rmxy = makef2_ff(resolution_xy * mx, resolution_xy * my); 

        color += size / mydistance( inxy,
            add_f2f2( rmxy ,
              makef2_ff( cos((0.0*120.0)/180.0*M_PI-time*speed) * r, 
                         sin((0.0*120.0)/180.0*M_PI-time*speed) * r) 
              )
        );

        color += size / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((1.0*120.0)/180.0*M_PI-time*speed) * r, 
                         sin((1.0*120.0)/180.0*M_PI-time*speed) * r ) )
            );

        color += size / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((2.0*120.0)/180.0*M_PI-time*speed) * r, 
                         sin((2.0*120.0)/180.0*M_PI-time*speed) * r ) )
            );

        color += size * 0.2 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((0.5*120.0)/180.0*M_PI-time*speed) * r * 0.5, 
                         sin((0.5*120.0)/180.0*M_PI-time*speed) * r * 0.5) )
            );

        color += size * 0.2 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((1.5*120.0)/180.0*M_PI-time*speed) * r * 0.5, 
                         sin((1.5*120.0)/180.0*M_PI-time*speed) * r * 0.5) )
            );

        color += size * 0.2 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((2.5*120.0)/180.0*M_PI-time*speed) * r * 0.5, 
                         sin((2.5*120.0)/180.0*M_PI-time*speed) * r * 0.5) )
            );

        color += size * 0.05 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((0.0*120.0)/180.0*M_PI-time*speed) * r * 0.2, 
                         sin((0.0*120.0)/180.0*M_PI-time*speed) * r * 0.2) )
            );
        color += size * 0.05 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((1.0*120.0)/180.0*M_PI-time*speed) * r * 0.2, 
                         sin((1.0*120.0)/180.0*M_PI-time*speed) * r * 0.2) )
            );
        color += size * 0.05 / mydistance( inxy, 
            add_f2f2( rmxy, 
              makef2_ff( cos((2.0*120.0)/180.0*M_PI-time*speed) * r * 0.2, 
                         sin((2.0*120.0)/180.0*M_PI-time*speed) * r * 0.2) )
            );

        return makef4_ffff( color * mColor.x,
                            color * mColor.y,
                            color * mColor.z,
                            1.0 ); 

}
