#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"
#include "gabor.h"

#include "api3d_bl.c"

/* FILTER_MODE_NONE */

float mymod( float n, float d ) { return d * fract_f( n / d ); }
float myintmod( float n, float d ) {
    return floor_f( d * fract_f( n / d ) + 0.5f );
}

float hash_noise( float x, float y ) {
    return gabor_noise( 1.0f, 0.05f, 20.0f, 0.0f, 16.0f, 256, x, y ) * 256.0f;
}

int Hash( __global global_buffers * g, float3 P )
{

  P = div_f3f( P, 256.0f );
  float2 texcoord;
  int ix;
  int iy;

  texcoord.x = P.z;
  texcoord.y = 0.0f;
  float x;
  //x = hash_noise( P.x, P.y );
  x = samplef_ff( g->g_txRand_f, g->sizex, g->sizey, P.x, P.y ) * 256.0f;
  float y;
  //y = hash_noise( P.z, 0.0f );
  y = samplef_ff( g->g_txRand_f, g->sizex, g->sizey, texcoord.x, texcoord.y ) * 256.0f;
  ix = x;
  iy = y;
  return ix+iy;  
}



float3 Grad3F(float x) {
    float3 ret;
    ret.x = mymod( x, 2.0f ) * 2.0f - 1.0f;
    ret.y = mymod( floor_f( x * 0.5f ), 2.0f ) - 1.0f;
    ret.z = 0.0f;
    float c = step_f( x, 7.0f );
    ret = add_f3f3(
        mul_f3f( ret, c ), mul_f3f( swizzle_f3zxy( ret ), 1.0f-c )
    );
    c = step_f( x, 3.0f );
    ret = add_f3f3(
        mul_f3f( ret, c ), mul_f3f( swizzle_f3xzy( ret ), 1.0f-c )
    );
    return ret;
}

// 4D case is more regular


// Basic 3D simplex noise


float Snoise3D( __global global_buffers * g, float3 P )
{
    float s = ( P.x + P.y + P.z ) * 0.333333333333f;
    float3 Pi = floor_f3( add_f3f( P, s ) );
    float t = ( Pi.x + Pi.y + Pi.z ) * 0.166666666667f;
    float3 P0 = add_f3f( Pi, - t );
    float3 Pf0 = sub_f3f3( P, P0 );

    float3 simplex_0, simplex_1, simplex_2, simplex_3;
    int i;
    float3 T = step_f3( swizzle_f3yxz( Pf0 ), swizzle_f3xzy( Pf0 ) );
    simplex_0 = makef3_f( 0.0f );
    simplex_1 = sub_ff3( 1.0f, step_f3( swizzle_f3xzy( T ), swizzle_f3yxz( T ) ) );
    simplex_2 = step_f3( swizzle_f3yxz( T ), swizzle_f3xzy( T ) );
    simplex_3 = makef3_f( 1 );


    float3 grad;
    float n;
    n = 0.0f;
    i = 0;
        // i = 0
        float3 Pf = add_f3f( sub_f3f3( Pf0, simplex_0 ), ( 0.166666666667f) * 0.0f );
        int h = Hash( g, add_f3f3( Pi, simplex_0 ) );
        float d = saturate_f( 0.6f - ( sq_f(Pf.x) + sq_f(Pf.y) + sq_f(Pf.z) ) );
        d = sq_f( d ); 
        grad = Grad3F( myintmod( h, 31.0f ) );
        n += sq_f( d ) * (grad.x * Pf.x + grad.y * Pf.y + grad.z * Pf.z);

        // i = 1
        float3 Pf = add_f3f( sub_f3f3( Pf0, simplex_1 ), ( 0.166666666667f) * 1.0f );
        int h = Hash( g, add_f3f3( Pi, simplex_1 ) );
        float d = saturate_f( 0.6f - ( sq_f(Pf.x) + sq_f(Pf.y) + sq_f(Pf.z) ) );
        d = sq_f( d ); 
        grad = Grad3F( myintmod( h, 31.0f ) );
        n += sq_f( d ) * (grad.x * Pf.x + grad.y * Pf.y + grad.z * Pf.z);

        // i = 2
        float3 Pf = add_f3f( sub_f3f3( Pf0, simplex_2 ), ( 0.166666666667f) * 2.0f );
        int h = Hash( g, add_f3f3( Pi, simplex_2 ) );
        float d = saturate_f( 0.6f - ( sq_f(Pf.x) + sq_f(Pf.y) + sq_f(Pf.z) ) );
        d = sq_f( d ); 
        grad = Grad3F( myintmod( h, 31.0f ) );
        n += sq_f( d ) * (grad.x * Pf.x + grad.y * Pf.y + grad.z * Pf.z);

        // i = 3
        float3 Pf = add_f3f( sub_f3f3( Pf0, simplex_3 ), ( 0.166666666667f) * 3.0f );
        int h = Hash( g, add_f3f3( Pi, simplex_3 ) );
        float d = saturate_f( 0.6f - ( sq_f(Pf.x) + sq_f(Pf.y) + sq_f(Pf.z) ) );
        d = sq_f( d ); 
        grad = Grad3F( myintmod( h, 31.0f ) );
        n += sq_f( d ) * (grad.x * Pf.x + grad.y * Pf.y + grad.z * Pf.z);

    return 32.0f * n;
}




float4
perlin(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c
) {
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );
   float4 lightDiffuse = makef4_f( 1.0f );
   float3 Pos = in->objPos;

#if 0
   float3 N = normalize_f3(in->worldNorm);
   float C = saturate_f(dot_f3( N, in->vLightDirVec) );
   float4 Diffuse = add_f4f4( Diffuse, mul_f4f( lightDiffuse, C ) );
#else
   float4 Diffuse = makef4_f( 1.0f );
#endif

    float octave3 = 0.25f * Snoise3D( g, mul_f3f( Pos, 8.0f ) );
    float octave2 = 0.5f * Snoise3D( g, mul_f3f( Pos, 4.0f ) );
    float octave1 = 1.0f * Snoise3D( g, mul_f3f( Pos, 2.0f ) );
    float octave0 = 2.0f * Snoise3D( g, mul_f3f( Pos, 1.0f ) );
   float noise = octave3 + octave2 + octave1 + octave0;
   noise = 0.5f + 0.5f * noise;

   float4 colorOut = mul_f4f( Diffuse, noise );
   return colorOut;
}

