#ifndef PRNG_H
#define PRNG_H

void
prng_bits( unsigned * st )
{
    unsigned old = *st;
    *st *= 3039177861u;
    *st += (*st == old);
}

float
prng_uniform_0_1( unsigned * st )
{
    prng_bits( st );
    return (*st) / (float) 0xffffffffu;
}

float
prng_uniform( unsigned * st, float min, float max )
{
    return min + prng_uniform_0_1( st ) * ( max - min );
}

unsigned
prng_poisson( unsigned * st, float mean )
{
    float g = exp( -mean );
    unsigned em = 0;
    float t = prng_uniform_0_1( st );
    while ( g < t ) {
        em += 1;
        t *= prng_uniform_0_1( st );
    }
    return em;
}

float2
prng_normal( unsigned * st, float mean, float sd )
{
    float u1 = prng_uniform_0_1( st );
    float u2 = prng_uniform_0_1( st );
    u1 = sqrt( -2.0f * log( u1 ) );
    u2 = 2 * M_PI * u2;
    return makef2_ff(
        mean + sd * u1 * sin( u2 ),
        mean + sd * u1 * cos( u2 )
    );
}

#endif
