#ifndef TEXTURE_VEC_SIZE
#error "cannot process texture.h without TEXTURE_VEC_SIZE"
#endif

#define TEXTURE_CAT(a,b) TEXTURE_XCAT(a,b)
#define TEXTURE_XCAT(a,b) a ## b

#define FLOAT_T TEXTURE_CAT(float, TEXTURE_VEC_SIZE)
#define F(prefix, suffix) TEXTURE_CAT(prefix, TEXTURE_CAT(TEXTURE_VEC_SIZE, suffix))
#define F1(op) TEXTURE_CAT(op, TEXTURE_CAT(_f, TEXTURE_CAT(TEXTURE_VEC_SIZE, f)))
#define F2(op) TEXTURE_CAT(op, TEXTURE_CAT(_f, TEXTURE_CAT(TEXTURE_VEC_SIZE, TEXTURE_CAT(f, TEXTURE_VEC_SIZE))))

#define makef_f(x) x
#define add_ff(x,y) ((x)+(y))
#define sub_ff(x,y) ((x)-(y))
#define mul_ff(x,y) ((x)*(y))
#define div_ff(x,y) ((x)/(y))

FLOAT_T
F(samplef, _ff_fractional)(
   __global FLOAT_T * tex, unsigned w, float u, float v
)
__attribute__((noinline))
{
    FLOAT_T p00, p01, p10, p11;
    float2 uv;
    int2 lo, hi;

    uv = makef2_ff( u - 1.0f, v - 1.0f );
    lo = int2_f2( floor_f2( uv ) );
    hi = int2_f2( ceil_f2( uv ) );

    p00 = p01 = p10 = F(makef,_f)( 0.0f );
    if ( lo.x > 0.0f && lo.y > 0.0f )
        p00 = tex[ lo.y * w + lo.x ];
    if ( lo.y > 0 )
        p01 = tex[ lo.y * w + hi.x ];
    if ( lo.x > 0 )
        p10 = tex[ hi.y * w + lo.x ];
    p11 = tex[ hi.y * w + hi.x ];

    return F1(lerp)(
        F1(lerp)( p00, p01, fract_f( uv.x ) ),
        F1(lerp)( p10, p11, fract_f( uv.x ) ),
        fract_f( uv.y )
    );
}

FLOAT_T
F(samplef,_ff_w)(
    __global FLOAT_T * tex, unsigned w, unsigned h,
    float u, float v, float du, float dv
)
__attribute((noinline))
{
    FLOAT_T a, b, c, d;
    float2 uv, duv, p0, p1, wraps, size;
    float weight;

    uv = makef2_ff( u, v );
    duv = makef2_ff( du, dv );
    p0 = sub_f2f2( uv, mul_f2f( abs_f2( duv ), 0.5f ) );
    p1 = add_f2f2( uv, mul_f2f( abs_f2( duv ), 0.5f ) );
    wraps = sub_f2f2( floor_f2( p1 ), floor_f2( p0 ) );

    size = makef2_ff( w, h );
    p0 = mul_f2f2( fract_f2( p0 ), size );
    p1 = mul_f2f2( fract_f2( p1 ), size );

    a = F(samplef,_ff_fractional)( tex, w, p0.x, p0.y );
    b = F2(add)(
        F(samplef,_ff_fractional)( tex, w, p1.x, p0.y ),
        F1(mul)( F(samplef,_ff_fractional)( tex, w, size.x, p0.y ), wraps.x )
    );
    c = F2(add)(
        F(samplef,_ff_fractional)( tex, w, p0.x, p1.y ),
        F1(mul)( F(samplef,_ff_fractional)( tex, w, p0.x, size.y ), wraps.y )
    );
    d = F2(add)(
        F2(add)(
            F(samplef,_ff_fractional)( tex, w, p1.x, p1.y ),
            F1(mul)( tex[ w * h - 1 ], wraps.x * wraps.y )
        ),
        F2(add)(
            F1(mul)( F(samplef,_ff_fractional)( tex, w, p1.x, size.y ), wraps.y ),
            F1(mul)( F(samplef,_ff_fractional)( tex, w, size.x, p1.y ), wraps.x )
        )
    );
    weight = du * dv * w * h;
    return F1(div)( F2(sub)( F2(sub)( d, c ), F2(sub)( b, a ) ), weight );
}

FLOAT_T
F(samplef,_ff)(
    __global FLOAT_T * tex, unsigned w, unsigned h, float u, float v
)
__attribute__((noinline))
{
    return F(samplef,_ff_w)( tex, w, h, u, v, 1.0f / w, 1.0f / h );
    /*
    FLOAT_T a, b, c, d;
    unsigned int i, j;

    i = fract_f( u ) * w;
    j = fract_f( v ) * h;

    a = b = c = F(makef,_f)( 0.0f );
    if ( i > 0 && j > 0 )
        a = tex[ (j-1) * w + (i-1) ];
    if ( j > 0 )
        b = tex[ (j-1) * w + i ];
    if ( i > 0 )
        c = tex[ j * w + (i-1) ];
    d = tex[ j * w + i ];
    return F2(sub)( F2(sub)( d, c ), F2(sub)( b, a ) );
    */
}

#undef makef_f
#undef add_ff
#undef sub_ff
#undef mul_ff
#undef div_ff

#undef FLOAT_T
#undef F
#undef F1
#undef F2
#undef TEXTURE_XCAT
#undef TEXTURE_CAT

