#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"

#include "api3d_bl.c"

/* https://www.shadertoy.com/view/ldl3W8 */ 

float
mylength( float2 pos )
{
    return sqrt(pos.x * pos.x + pos.y * pos.y); 
}

float 
myfract( float x ) 
{
        return x - floor(x) ; 
} 

float2 
myfract_f2( float2 x ) 
{
  return makef2_ff(
      myfract(x.x),
      myfract(x.y)); 
} 

#define vec3 float3
#define vec2 float2

vec3 mix_f3(vec3 x, vec3 y, float a) {
  // the product of x and (1 - a) plus the product of y and a. 
  // component-wise
  return makef3_fff(
      x.x * (1.0 - a) + y.x * a, 
      x.y * (1.0 - a) + y.y * a, 
      x.z * (1.0 - a) + y.z * a); 
} 

float mix_f1(float x, float y, float a) {
  return x * (1.0 - a) + y * a;
} 

vec2 sin_f2(vec2 a)
{
  return makef2_ff(sin(a.x), sin(a.y)); 
} 

vec2 hash2( vec2 p )
{
  // procedural white noise   
  return myfract_f2(
    mul_f2f(
      sin_f2 (
        makef2_ff(
          dot_f2(p,makef2_ff(127.1,311.7)),
          dot_f2(p,makef2_ff(269.5,183.3))
          )
        ),43758.5453));
}

vec3 voronoi( vec2 x )
{
  vec2 n ;
  vec2 f = myfract_f2(x);
  int i, j;

  //----------------------------------
  // first pass: regular voronoi
  //----------------------------------
  vec2 mg, mr;
  float md = 8.0;

  n.x = floor_f(x.x); 
  n.y = floor_f(x.y); 

  for( j=-1; j<=1; j++ ) {
    for( i=-1; i<=1; i++ )
    {
      vec2 g = makef2_ff((float)i,(float)j);
      vec2 o = hash2( add_f2f2(n , g) );
      vec2 r = add_f2f2(g , sub_f2f2(o , f));
      float d = dot_f2(r,r);

      if( d<md ) 
      {
        md = d;
        mr = r;
        mg = g;
      }
    }
  } 
  // return makef3_fff(md, mr.x, mr.y); 

  //----------------------------------
  // second pass: distance to borders
  //----------------------------------
  md = 8.0;
  for( j=-2; j<=2; j++ )
    for( i=-2; i<=2; i++ )
    {
      vec2 g = makef2_ff((float)i + mg.x,(float)j + mg.y);
      vec2 o = hash2( add_f2f2(n , g ));
      vec2 r = add_f2f2(g , sub_f2f2(o , f));
      vec2 mrmr = sub_f2f2(mr,r); 

      // md = min( md, dot( 0.5*(mr+r), normalize(r-mr) ) );

      if( dot_f2(mrmr,mrmr)>0.00001 ) {
        float2 t1 = mul_f2f( add_f2f2(mr,r), 0.5); 
        float2 t2 = normalize_f2(sub_f2f2(r,mr)); 
        md = fmin( md, dot_f2(t1, t2)); 
      } 
    }

  return makef3_fff(md, mr.x, mr.y) ; 
}

float4
voronoid(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c) 
{
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );
    float2 p = makef2_ff(in->objPos.x * 1.0 , in->objPos.y * 1.0); 

    vec3 c = voronoi( mul_f2f(p,8.0) );
    // return makef4_ffff( c.x, c.y, c.z, 1.0); 

    // isolines
    vec3 col ;
    col.x = c.x*(0.5 + 0.5*sin(64.0*c.x));
    col.y = c.x*(0.5 + 0.5*sin(64.0*c.x));
    col.z = c.x*(0.5 + 0.5*sin(64.0*c.x));
    // borders  
    col = mix_f3( makef3_fff(1.0,0.6,0.0), col, smoothstep_f( 0.04, 0.07, c.x ) );
    return makef4_ffff( col.x, col.y, col.z, 1.0); 
    // feature points
    float dd = mylength( makef2_ff(c.y, c.z) );
    col = mix_f3( makef3_fff(1.0,0.6,0.1), col, smoothstep_f( 0.0, 0.12, dd) );
    col = add_f3f3(col, 
          mul_f3f( makef3_fff(1.0,0.6,0.1),(1.0-smoothstep_f( 0.0, 0.04, dd))));

    return makef4_ffff( col.x, col.y, col.z, 1.0); 
}
