#include <math.h>

#include "api3d.h"
#include "factors.h"
#include "procedural_shaders.h"
#include "procedural_helpers.h"
#include "reprojection.h"
#include "prng.h"

#include "api3d_bl.c"

/* http://glslsandbox.com/e#24335.0 */

float time = 1.0;
const int maxIter = 70;

float
mylength( float2 pos )
{
    return abs_f( pos.x ) + abs_f( pos.y );
}

float2 cmult(float2 a, float2 b){
        return makef2_ff( a.x*b.x-a.y*b.y,
                          a.x*b.y+a.y*b.x );                
}

float2 iterate(float2 p, float2 c) {
  float2 p2 = cmult(p, p);
  float time = 3e-1 / mylength(p); 
  float2 tanp2 = makef2_ff( tan(p2.x), tan(p2.y) );
  return add_f2f2(sub_f2f2(tanp2, makef2_ff(0.52, 0.52)),
        makef2_ff(0.1 * cos(time), 0.1 * sin(time))); 
} 

int checkAbort(float2 p, float2 c) {
  return mylength(p) > 30.0 ; 
} 


float4 color(int iterations, float2 p) {
  float l2 = log(2.);
  float col = .75-((float)(iterations) - log(log(mylength(p)))/l2 )/ (float)(maxIter);
  return makef4_ffff(1.0 - col*1.2, 1.0 - col*1.1, 1.0 - col,1.);
}

float4
whorl(
    __global global_buffers * g,
    VS_OUTPUT * in,
    float2 offset,
    __global float * c) 
{
    axis_align_widths( &( in->dwdx ), &( in->dwdy ) );

    float2 c = makef2_ff((in->objPos.x) * 3 - 0.5, 
                         (in->objPos.y) * 3 - 0.0) ; 

    float2 p = c;
                
    for(int i = 0; i < maxIter ;i++) {
      p = iterate(p,c);
      if (checkAbort(p,c)) {        
        return color(i,p); 
      }
    }
    return (makef4_ffff(0.05, 0.025, 0.0, 1.0)); 
}
