#ifndef _OPTIMIZER_HOOKE_H_
#define _OPTIMIZER_HOOKE_H_

#include "optimizer.h"

#define HOOKE_DEFAULT_EPSILON  1.0e-7      // halting cirterion
#define HOOKE_DEFAULT_RHO      0.5         // between 0.0 (aggressive, fast) and 1.0 (accurate, slow)

class OptimizerHooke : public Optimizer
{
 public:
  OptimizerHooke(ErrorFunk funk, void* funk_params, float* start_pos, int ndim, float rho=HOOKE_DEFAULT_RHO, float epsilon=HOOKE_DEFAULT_EPSILON);

  ~OptimizerHooke(void);

  void reset(float* start_pos);
  int numdim(void) const { return ndim; }
  float* pos(void) const;
  bool step(void);
  float bestError(void) const { return error; }
  int solve(int max_iter=OPTIMIZER_MAX_ITER);

  
 private:
  void bestNearby(void);

  float* current_pos;
  float current_error;
  float* end_pos;
  float error;
  ErrorFunk funk;
  void* funk_param;
  int ndim;

  float * delta;
  float steplength;
  float rho, epsilon;
};


#endif /* _OPTIMIZER_HOOKE_H_ */

