#ifndef _OPTIMIZER_H_
#define _OPTIMIZER_H_

#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>

typedef float(*ErrorFunk)(float*, void*);

const int OPTIMIZER_MAX_ITER = 10000;

class Optimizer 
{
public:
  virtual void reset(float *)=0;
  virtual float* pos()const=0;
  virtual int numdim() const=0;
  virtual bool step()=0;
  virtual float bestError() const=0;
  virtual int solve(int max_iter=OPTIMIZER_MAX_ITER)=0;
};

class Timer {
  struct timeb tb_before;
 public:
  Timer();
  ~Timer();
  void start(void);
  double stop(void);
};

inline
Timer::Timer ()
{
}

inline
Timer::~Timer ()
{
}

inline void
Timer::start ()
{
  ftime((struct timeb *)&tb_before);
}

inline double
Timer::stop ()
{
  struct timeb tb_after;

  // Get timestamp
  ftime((struct timeb *)&tb_after);

  // Compute elapsed time
  long secs_elapsed = tb_after.time - tb_before.time;
  long msecs_elapsed = tb_after.millitm - tb_before.millitm;
  if (msecs_elapsed < 0)
    {
      secs_elapsed--;
      msecs_elapsed+=1000;
    }

  double ret = ( ((double)secs_elapsed) * 1000.0 + ((double)msecs_elapsed) ) / 1000.0;

  return ret;
}

#endif /* _OPTIMIZER_H_ */
