#ifndef _OPTIMIZER_SIMPLEX_H_
#define _OPTIMIZER_SIMPLEX_H_

#include "optimizer.h"

#include <set>

const float SIMPLEX_TOL = 1e-6f;

class OptimizerSimplex : public Optimizer
{
public:
    OptimizerSimplex(ErrorFunk _funk, void* _funk_params, float* start_pos,
		     int ndim);
    OptimizerSimplex(ErrorFunk _funk, void* _funk_params, float* start_pos,
		     int ndim, float* steps);
    OptimizerSimplex(ErrorFunk _funk, void* _funk_params, float* start_pos,
		     int ndim, float stepSize);
    
    ~OptimizerSimplex(); 

    void reset(float *start_pos);

    // get number of error_evals so far
    unsigned long numEvals(void) { return num_evals; }

    // get number of simplex steps so far
    unsigned long numSteps(void) { return num_steps; }

    // return number of dimensions of simplex 
    inline int numdim(void) const { return ndim; }
    
    // creates *NEW COPY* of position and returns it 
    float* pos(void) const;  
    
    // make one simplex change return true if found minimum, else false
    bool step(void);  

    float bestError(void) const { return errors[best]; }
   
    // run until convergence or some maximum number of iterations  
    int solve(int max_iter = OPTIMIZER_MAX_ITER);
    
private:
    void logEval(float error, float besterr);
    Timer timer;
    FILE* logfile;
    OptimizerSimplex() {}
    void findBestWorst();
    float mutateSimplex(float factor);
    inline void sumPositions();
    inline float* arraydup(const float* a, int ndim) const;

    std::set< float > visited;
    unsigned long num_steps;
    unsigned long last_reset_step;
    unsigned long num_evals;
    int inf_count;
    int ndim;
    int best;
    int worst;
    int nworst;
    float** positions;
    float*  errors;
    float*  sum_positions;
    float*  mutate_position;
    ErrorFunk funk;
    void* funk_params;
};



//
// a few helper functions
//

inline float*
OptimizerSimplex::arraydup(const float* a, int ndim)const
{
    float* ret_array = new float[ndim];
    for (int i = 0; i < ndim; i++)
        ret_array[i] = a[i];
    return ret_array;
}

inline void
OptimizerSimplex::sumPositions()
{
  for (int i = 0; i < ndim; i++)
    sum_positions[i] = 0.0f;
  
  for (int i = 0; i <= ndim; i++)
    for (int j = 0; j < ndim; j++)
      sum_positions[j] += positions[i][j];
}


#endif // _OPTIMIZER_SIMPLEX_H_
