#!/bin/sh

if [ $# -lt 1 ]; then
    echo "Usage: $0 shader [--msaa] [cl-file]"
    exit
fi

root=`dirname "$0"`
root=`cd "$root" ; pwd`
root=`dirname "$root"`

shader=$1
msaa=
clfile=
delcl=false
shift

while [ $# -gt 0 ] ; do
    case "$1" in
      --msaa) msaa="--msaa 16";;
      *.cl)   clfile=$1 ;;
      *)      echo "unrecognized argument '$1'" ; exit 1;;
    esac
    shift
done

if [ "X${clfile:+set}" != "Xset" ] ; then
    tmp=`mktemp --suffix .c`
    gcc -E "$root/src/$shader.c" -DFILTER_MODE_TENT -o $tmp
    clfile=`mktemp --suffix .cl`
    "$root/tools/compile.sh" $shader $clfile $tmp
    rm -f $tmp
    delcl=true
fi

"$root/tools/test.py" \
    --perspective \
    --imagew 1 \
    --timew 1 \
    --low-var 0.05 \
    --multifit \
    --timeout 240 \
    --zoom 0,30,45,60,90 \
    ${msaa} \
    $shader $clfile /dev/tty 256x256

if $delcl ; then
    echo "removing $clfile"
    rm $clfile
fi

