#!/bin/sh

do_msaa=true
do_orig=true
do_search=true
do_target=true

case "$1" in
  -h|--help)
    echo "Usage: $0 [--no-msaa|--no-orig|--no-search|--no-target] [shader...]"
    exit
    ;;
  "")
    shaders="
        hardstep
        ridges
        pulse
        checker
        circles
        gabor_brick
        gabor_marble
        gabor_rust
        gabor_wood
        gabor_harlequin
        perlin
    " ;;
  *)
    shaders=""
    for arg in "$@" ; do
        case "$arg" in
          --no-msaa)   do_msaa=false ;;
          --no-orig)   do_orig=false ;;
          --no-search) do_search=false ;;
          --no-target) do_target=false ;;
          *) shaders="$shaders $arg" ;;
        esac
    done
esac

if $do_search ; then
    missing=0
    for shader in $shaders ; do
        if [ ! -r $shader-simplex.cl ] ; then
            echo "Missing $shader-simplex.cl"
            missing=`expr $missing + 1`
        fi
    done
    if [ $missing -gt 0 ] ; then
        echo "missing $missing shaders. Exiting"
        exit 1
    fi
fi

root=`dirname "$0"`
root=`cd "$root" ; pwd`
root=`dirname "$root"`

for shader in $shaders ; do
    echo "===== $shader ====="

    if $do_search ; then
        echo "search"
        "$root/tools/animate2.py" \
            --size 256x256 \
            --samples 1 \
            --fps right \
            -o movie/$shader-search \
            0 360 200 "our result" $shader-simplex.cl
    fi

    tmp=`mktemp --suffix .c`
    gcc -E src/$shader.c -DFILTER_MODE_TENT -o $tmp

    if $do_target ; then
        echo "target"
        "$root/tools/animate2.py" \
            --size 256x256 \
            --samples 2048 \
            --fps left \
            -o movie/$shader-target \
            0 360 200 target $shader.c
    fi
    if $do_msaa ; then
        echo "msaa"
        "$root/tools/animate2.py" \
            --size 256x256 \
            --samples 16 \
            --fps right \
            -o movie/$shader-msaa \
            0 360 200 MSAA-16 $shader.c
    fi
    if $do_orig ; then
        echo "original"
        "$root/tools/animate2.py" \
            --size 256x256 \
            --samples 1 \
            --fps right \
            -o movie/$shader-original \
            0 360 200 original $shader.c
    fi

    rm $tmp
done

