#!/bin/sh

if [ $# -lt 5 ] ; then
    echo "Usage: $0 outfile shader exe-name source-name distances size"
    exit
fi

root=`dirname "$0"`
root=`cd "$root" ; pwd`

timeout=10
outfile=$1
shader=$2
exe=$3
src=$4
ndist=$5
size=$6

case "$exe" in
    */*) : ;;
    *) exe="./$exe" ;;
esac
if type diff-img > /dev/null 2>&1 ; then :
else
    type diff-img 1>&2
    exit 2
fi

get_golden()
{
    if [ "X${do_freq:+set}" = "Xset" ] ; then
        echo "$root/../golden/$shader-$size-$ss-freq.ppm"
    else
        echo "$root/../golden/$shader-$size-$ss.ppm"
    fi
}

"$root/compile.sh" "$shader" "$exe" "$src" || exit $?

no_orig=true
ss=1
for i in `seq $ndist` ; do
    golden="`get_golden`"
    if [ ! -f "$golden" ] ; then
        if $no_orig ; then
            "$root/compile.sh" \
                "$shader" \
                "$root/../$shader.exe" \
                "$root/../src/$shader.c"
            no_orig=false
        fi
        echo "generating golden image ($size @ $ss)" 1>&2
        mkdir -p "$root/../golden"
        "$root/../$shader.exe" \
            --subsamples $ss \
            --zoom $ss \
            --size "$size" \
            "$golden"
    fi
    ss=`expr $ss + $ss`
done
if $no_orig ; then : 
else
    rm "$root/../$shader.exe"
fi

set x
shift

rows=1
for i in `seq $ndist` ; do
    ss=`expr $ss / 2`
    golden="`get_golden`"
    tmpfile=`mktemp --suffix .ppm`
    timeout $timeout \
    "$exe" \
        --subsamples 1 \
        --zoom $ss \
        --size "$size" \
        "$tmpfile"
    if [ -r $tmpfile ] ; then
        diffssim=`mktemp --suffix .ppm`
        dft1=`mktemp --suffix .ppm`
        dft2=`mktemp --suffix .ppm`
        f=`diff-img "$golden" "$tmpfile" --dft "$dft1" "$dft2" -o "$diffssim" | grep avg | cut -d\  -f4`
        set x -label "" "$dft2" "$@" ; shift
        set x -label "$f" "$diffssim" "$@" ; shift
        set x -label "" "$dft1" "$@" ; shift

        set x -label "" "$tmpfile" "$@" ; shift

        diffl2=`mktemp --suffix .ppm`
        f=`diff-img "$golden" "$tmpfile" -o "$diffl2" | grep avg | cut -d\  -f4`
        set x "$golden" -label "$f" "$diffl2" "$@" ; shift
    fi
    rows=`expr $rows + 1`
done

width=`echo $size | cut -dx -f1`
for label in "learned (DFT)" "diff (DFT)" "golden (DFT)" learned "diff (L2)" golden ; do
    tmpfile=`mktemp --suffix .ppm`
    convert -size ${width}x20 \
            -draw "scale 1.5,1.5 gravity Center text 1x1 '$label'" \
            canvas:white \
            $tmpfile
    set x "$tmpfile" "$@"
    shift
done
montage -tile 6x$rows -mode Concatenate -frame 2x2 \
        "$@" "$outfile"

rm -f "$1" "$2" "$3" "$4" "$5" "$6"
shift ; shift ; shift ; shift ; shift ; shift
while [ $# -gt 0 ] ; do
    rm -f "$4"
    shift ; shift ; shift ; shift
    rm -f "$3"
    shift ; shift ; shift
    rm -f "$3"
    shift ; shift ; shift
    rm -f "$3"
    shift ; shift ; shift
    rm -f "$3"
    shift ; shift ; shift
done

