open Cil

let label_counter = ref 0

let nextLabel name =
  incr label_counter;
  Label(Printf.sprintf "__inline_%s_%d" name !label_counter, locUnknown, false)

class inlineBlockTransform retvar name =
object
  inherit nopCilVisitor

  val target = {dummyStmt with labels = [nextLabel name]; skind = Instr([])}

  method vfunc fd =
    ChangeDoChildrenPost(fd, fun fd ->
      fd.sbody.bstmts <- List.rev (target :: List.rev fd.sbody.bstmts);
      fd
    )
    
  method vblock b =
    ChangeDoChildrenPost(b, fun b ->
      let stmts =
        List.map (fun stmt ->
          match stmt.skind with
          | Return(Some(e), loc) -> [
              {stmt with skind = Instr([Set(var retvar, e, loc)])};
              mkStmt (Goto(ref target, locUnknown))
            ]
          | Return(None, loc) -> [mkStmt (Goto(ref target, locUnknown))]
          | Instr([]) -> []
          | Instr([_]) -> [stmt]
          | Instr(first::rest) ->
            stmt.skind <- Instr([first]);
            stmt :: List.map (fun i -> mkStmtOneInstr i) rest
          | Block(b') when (List.length b'.battrs) = 0 ->
            b'.bstmts
          | _ -> [stmt]
        ) b.bstmts
      in
      {b with bstmts = List.flatten stmts}
    )
end

class inliner ftbl =
object
  inherit nopCilVisitor

  val stack = Hashtbl.create 11

  val mutable this_fd = dummyFunDec

  method vfunc fd =
    let previous = this_fd in
    this_fd <- fd;
    ChangeDoChildrenPost(fd, fun fd -> this_fd <- previous; fd)

  method vstmt s =
    match s.skind with
    | Instr([Call(lv, Lval(Var(vi), NoOffset), args, loc)])
        when (Hashtbl.mem ftbl vi.vname) && not (Hashtbl.mem stack vi.vname) ->
      let rt, _, is_vararg, _ = splitFunctionType vi.vtype in
      if is_vararg then
        DoChildren
      else begin
        Hashtbl.add stack vi.vname ();

        let retvar =
          makeTempVar this_fd ~name:(Printf.sprintf "__return_%s" vi.vname) rt
        in
        let fd = copyFunction (Hashtbl.find ftbl vi.vname) "copy" in
        let fd =
          visitCilFunction (new inlineBlockTransform retvar vi.vname) fd
        in

        let block = fd.sbody in
        let _ =
          match lv with
          | Some(lv) ->
            let assignment =
              mkStmtOneInstr (Set(lv, Lval(var retvar), locUnknown))
            in
            block.bstmts <- List.rev (assignment :: List.rev block.bstmts)
          | _ -> ()
        in
        block.bstmts <-
          List.map2 (fun fvi arg ->
            let tmp =
              makeTempVar this_fd ~insert:false ~name:fvi.vname fvi.vtype
            in
            fvi.vname <- tmp.vname;
            this_fd.slocals <- this_fd.slocals @ [fvi];

            mkStmtOneInstr (Set(var fvi, arg, locUnknown))
          ) fd.sformals args @ block.bstmts;

        List.iter (fun lvi ->
          let tmp =
            makeTempVar this_fd ~insert:false ~name:lvi.vname lvi.vtype
          in
          lvi.vname <- tmp.vname;
          this_fd.slocals <- this_fd.slocals @ [lvi]
        ) fd.slocals;

        let s' = {s with skind = Block(block)} in
        ChangeDoChildrenPost(s', fun s -> Hashtbl.remove stack vi.vname; s)
      end
    | _ -> DoChildren
end

let inline fd func_defs =
  visitCilFunction (new inliner func_defs) fd

