open Cil
open Pretty
open Utils

module type WidthAnalysis =
sig
  type t

  val init : bool -> Cil.file -> unit

  val initStartData : int -> Cil.varinfo list -> unit

  val compute : Cil.stmt list -> unit

  val getData : int -> t

  val transformExp : t -> Cil.exp -> Cil.exp option

  val transformFunc : t -> string -> Cil.lval option -> Cil.exp list -> Cil.instr list option
end

module Make (WA : WidthAnalysis) =
struct

class transformVisitor dostmts getstmts =
object
  inherit nopCilVisitor

  val mutable current_fd = dummyFunDec

  method vfunc fd =
    let first = List.hd fd.sbody.bstmts in
    WA.initStartData first.sid fd.sformals;
    WA.compute [first];

    let old_fd = current_fd in
    current_fd <- fd;
    ChangeDoChildrenPost(fd, fun fd -> current_fd <- old_fd; fd)

  method vstmt s =
    let data = WA.getData s.sid in
    let apply_instrs callback =
      let ins =
        try callback () with Failure msg -> failure "%t: %s" d_thisloc msg
      in
      match ins with
      | Some(ins) ->
        getstmts := ISet.add s.sid !getstmts;
        if ISet.mem s.sid dostmts then
          s.skind <- Instr(ins)
      | None -> ()
    in

    let _ =
      match s.skind with
      | Instr([]) -> ()
      | Instr([Set(lv, e, loc)]) ->
        apply_instrs (fun () ->
          match WA.transformExp data e with
          | Some(e') -> Some([Set(lv, e', loc)])
          | None     -> None
        )
      | Instr([Call(rv, Lval(Var(vi),NoOffset), args, loc)]) ->
        apply_instrs (fun () -> WA.transformFunc data vi.vname rv args)
      | Return(_) -> ()
      | _ -> failure "%t: don't know how to bandlimit: %a" d_thisloc dn_stmt s
    in
    DoChildren
end

let init debug cilfile = WA.init debug cilfile

let apply fd dostmts =
  let getstmts = ref ISet.empty in
  let _ = visitCilFunction (new transformVisitor dostmts getstmts) fd in
  !getstmts

end

