module Make (Ord : Map.OrderedType) =
struct

module OrdMap = Map.Make(Ord)

type key = Ord.t

type 'a t = Trie of 'a t OrdMap.t * 'a option

let empty = Trie(OrdMap.empty, None)

(*
let debug m =
  let rec helper indent index (Trie(map, value)) =
    begin match value with
    | Some(value) -> Printf.printf "%s[%s] (%s)\n" indent index value
    | None        -> Printf.printf "%s[%s]\n" indent index
    end;
    OrdMap.iter (fun i m ->
      helper (indent ^ "  ") (string_of_int i) m
    ) map
  in
  helper "" "" m
*)

(**/**)
let rec unzip ?(create=false) crumbs x m =
  match x, m with
  | hd::tl, Trie(map, value) ->
    let child =
      try
        (OrdMap.find) hd map
      with Not_found ->
        if create then empty else raise Not_found
    in
    unzip ~create ((hd, map, value)::crumbs) tl child
  | [], Trie(map, value) ->
    crumbs, map, value

let rec zip crumbs map value =
  if (value = None) && ((OrdMap.is_empty) map) then
    match crumbs with
    | (hd, map', value')::crumbs ->
      zip crumbs (OrdMap.remove hd map') value'
    | [] -> empty
  else
    let child = Trie(map, value) in
    match crumbs with
    | (hd, map', value')::crumbs ->
      zip crumbs ((OrdMap.add) hd child map') value'
    | [] -> child
(**/**)

let mem x m =
  try
    match unzip [] x m with
    | _, _, Some(_) -> true
    | _, _, None    -> false
  with Not_found -> false

let find x m =
  match unzip [] x m with
  | _, _, Some(value) -> value
  | _, _, None -> raise Not_found

let add x y m =
  let crumbs, map, value = unzip ~create:true [] x m in
  zip crumbs map (Some(y))

let remove x m =
  try
    let crumbs, map, value = unzip [] x m in
    zip crumbs map None
  with Not_found -> m

let singleton x y = add x y empty

let fold f m a =
  let rec helper path m a =
    match m with
    | Trie(map, value) ->
      let a =
        match value with
        | Some(value) -> f (List.rev path) value a
        | None -> a
      in
      OrdMap.fold (fun i m a -> helper (i::path) m a) map a
  in
  helper [] m a

let iter f m = fold (fun k v _ -> f k v) m ()

let size m = fold (fun _ _ n -> n + 1) m 0

let find_prefix x m =
  let _, map, value = unzip [] x m in
  List.rev (fold (fun k v lst -> (k, v) :: lst) m [])
end

