#!/usr/bin/python2

from glob import glob
from math import ceil, log10
from optparse import OptionParser
import os
from subprocess import Popen, PIPE, call, check_call
import sys

from utils import *

parser = OptionParser( usage = "%prog [options] movie-dir outdir" )
parser.add_option(
    "--skip", metavar = "shader", action = "append", default = list(),
    help = "skip the named shader. May be used multiple times"
)
parser.add_option(
    "--title", metavar = "file,seconds",
    help = "title slide and the number of seconds it should appear"
)
options, args = parser.parse_args()

if len( args ) < 2:
    parser.print_help()
    exit()

movies = args[ 0 ]
outdir = args[ 1 ]

if options.title is not None:
    options.title = options.title.split( ",", 1 )
    options.title[ 1 ] = int( float( options.title[ 1 ] ) * 25 )

#if os.path.exists( outdir ):
#    check_call( [ "rm", "-rf", outdir ] )
#os.makedirs( outdir )

shaders = [
    ( "hardstep",        "step" ),
    ( "ridges",          "ridges" ),
    ( "pulse",           "pulse" ),
    ( "gabor_rust",      "noise1" ),
    ( "checker",         "checker" ),
    ( "circles",         "circles1" ),
    ( "gabor_wood",      "wood" ),
    ( "gabor_brick",     "brick" ),
    ( "gabor_marble",    "noise2" ),
    ( "gabor_harlequin", "circles2" ),
    ( "perlin",          "perlin" ),
]
comparison = [
    "original",
    "msaa",
    "search",
]

def get_frames( movies, shader, mode ):
    return sorted( glob( "%s/%s-%s/*" % ( movies, shader, mode ) ) )

def split_frame( ctx, fname, side ):
    tmpfile = ctx.append( mktemp( ".png" ) )

    cmd = [ "convert", fname, "-set", "colorspace", "RGB" ]
    if side == "left":
        cmd += [ "-gravity", "West" ]
    else:
        cmd += [ "-gravity", "East" ]
    cmd += [ "-crop", "50x100%+0+0", "+repage", tmpfile ]
    check_call( cmd )
    return tmpfile

nframes = 0
for shader, name in shaders:
    for mode in comparison:
        nframes += len( get_frames( movies, shader, mode ) )
fmt = "%s/frame-%%0%dd.png" % ( outdir, int( ceil( log10( nframes ) ) ) )

next_frame = 0
if options.title is not None:
    print >>sys.stderr, "==== title ===="
    for i in range( options.title[ 1 ] ):
        outname = fmt % next_frame
        cmd = [ "convert", options.title[ 0 ], outname ]
        check_call( cmd )
        next_frame += 1
for shader, name in shaders:
    print >>sys.stderr, "====", shader, "===="
    golden = list()
    with ContextList() as ctx:
        for frame in get_frames( movies, shader, "target" ):
            golden.append( split_frame( ctx, frame, "left" ) )
        with ContextList() as tmp:
            for mode in comparison:
                print >>sys.stderr, mode
                frames = get_frames( movies, shader, mode )
                for lhs, rhs in zip( golden, frames ):
                    if shader not in options.skip and name not in options.skip:
                        rhs = split_frame( tmp, rhs, "right" )
                        outname = fmt % next_frame
                        with mktemp( ".png" ) as joined:
                            cmd = [ "montage",
                                "-set", "colorspace", "RGB",
                                "-tile", "2x1",
                                "-mode", "Concatenate",
                                lhs, rhs, joined
                            ]
                            check_call( cmd )
                            cmd = [ "convert", joined,
                                "-gravity", "North",
                                "-stroke", "black",
                                "-fill", "white",
                                "-pointsize", "25",
                                "-draw", "text 0,0 '%s'" % name,
                                outname
                            ]
                            check_call( cmd )
                    next_frame += 1
                    if next_frame % 100 == 0:
                        print >>sys.stderr, next_frame, "frames"

