#include <stdlib.h>

#include "globals.h"
#include "procedural_shaders.h"

void TEXPROC ( global_buffers * g, VS_OUTPUT *, float2, float4 * );

const char *
get_kernel_source()
{
    return kernel_source;
}

float4 *
plot(
    global_buffers * g,
    unsigned int * raster_w,
    unsigned int * raster_h,
    unsigned int subpixels,
    float scale_factor,
    int expand,
    unsigned int grayscale
) {
    VS_OUTPUT out;
    float4 * raster;
    float4 color, c;
    float2 offset, o;
    float scale, step = 1.0f;
    unsigned int i, j, base, x, y;

    scale = scale_factor / 255.0f;
    o = makef2_ff(
        - scale * (*raster_w) * 0.5f,
        - scale * (*raster_h) * 0.5f
    );
    offset = makef2_ff( 0.0f, 0.0f );

    if ( expand ) {
        (*raster_w) *= subpixels;
        (*raster_h) *= subpixels;
        step /= (float) subpixels;
        subpixels = 1;
    }
    out.dwdx = makef3_fff( scale / subpixels, 0.0f, 0.0f );
    out.dwdy = makef3_fff( 0.0f, scale / subpixels, 0.0f );

    raster = (float4 *) malloc( sizeof( float4 ) * (*raster_w) * (*raster_h) );
    for ( j = 0, base = 0; j < (*raster_h); ++j, base += (*raster_w) ) {
        for ( i = 0; i < (*raster_w); ++i ) {
            color = makef4_f( 0.0f );
            for ( y = j * subpixels; y < ( j + 1 ) * subpixels; ++y ) {
                for ( x = i * subpixels; x < ( i + 1 ) * subpixels; ++x ) {
                    out.worldPos  = makef3_fff(
                        o.x + x * out.dwdx.x * step,
                        o.y + y * out.dwdy.y * step,
                        0.1f
                    );
                    out.objPos    = out.worldPos;
                    out.worldNorm = makef3_fff( 0.0f, 0.0f, 1.0f );
                    TEXPROC( g, &out, offset, &c );
                    color = add_f4f4( color, c );
                }
            }
            raster[ base + i ] = div_f4f( color, subpixels * subpixels );
            if ( grayscale )
                raster[ base + i ] = makef4_f( dot_f4(
                     raster[ base + i ],
                     makef4_ffff( 0.2126, 0.7152, 0.0722, 0.0 )
                ) );
        }
    }

    return raster;
}

