#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pam.h>

#ifdef PAM_STRUCT_SIZE
#  define LENGTH_OF_PAM PAM_STRUCT_SIZE( tuple_type )
#else
#  define LENGTH_OF_PAM sizeof( struct pam )
#endif

static int pam_init = 0;

float *
load_ppm(
    const char * fname,
    unsigned int * w,
    unsigned int * h,
    unsigned int * depth
) {
    struct pam metadata;
    FILE * fp;
    tuple * row;
    float * raster;
    float scale;
    unsigned int x, y, c, i;

    if ( ! pam_init ) {
        pm_init( "pnm", 0 );
        pam_init = 1;
    }

    fp = fopen( fname, "rb" );
    pnm_readpaminit( fp, &metadata, LENGTH_OF_PAM );

    scale = 1.0f / (float) metadata.maxval;
    *w = metadata.width;
    *h = metadata.height;
    *depth = metadata.depth;

    raster = (float*) malloc( sizeof( float ) * (*w) * (*h) * (*depth) );
    row = pnm_allocpamrow( &metadata );
    for ( y = 0, i = 0; y < *h; ++y ) {
        pnm_readpamrow( &metadata, row );
        for ( x = 0; x < *w; ++x )
            for ( c = 0; c < *depth; ++c, ++i )
                raster[ i ] = row[ x ][ c ] * scale;
    }
    pnm_freepamrow( row );

    fclose( fp );

    return raster;
}

void
save_ppm(
    const char * fname,
    float * raster,
    unsigned int w,
    unsigned int h,
    unsigned int depth
) {
    struct pam metadata;
    FILE * fp;
    tuple * row;
    unsigned int x, y, c, i;
    int samp;

    if ( ! pam_init ) {
        pm_init( "pnm", 0 );
        pam_init = 1;
    }

    metadata.size   = sizeof( metadata );
    metadata.len    = LENGTH_OF_PAM;
    metadata.height = h;
    metadata.width  = w;
    metadata.maxval = 255;
    if ( depth == 1 ) {
        metadata.format = RPGM_FORMAT;
        metadata.depth  = depth;
    } else {
        metadata.format = RPPM_FORMAT;
        metadata.depth  = 3;
    }

    fp = fopen( fname, "wb" );
    metadata.file = fp;

    pnm_writepaminit( &metadata );
    row = pnm_allocpamrow( &metadata );
    for ( y = 0, i = 0; y < h; ++y ) {
        for ( x = 0; x < w; ++x ) {
            for ( c = 0; c < depth && c < metadata.depth; ++c, ++i ) {
                samp = raster[ i ] * 255;
                samp = samp < 0 ? 0 : 255 < samp ? 255 : samp;
                row[ x ][ c ] = samp;
            }
            i += metadata.depth - c;
        }
        pnm_writepamrow( &metadata, row );
    }
    pnm_freepamrow( row );

    fclose( fp );
}

