#!/bin/sh

if [ $# -lt 1 ] ; then
    echo "Usage: $0 shader"
    exit
fi

root=`dirname "$0"`
root=`cd "$root" ; pwd`
root=`dirname "$root"`

shader=$1

pareto=/home/jad5ju/Projects/shader_simplification/pareto
for seed in `seq 1 10` ; do
    echo ===== genetic search: $shader $seed =====
    failure_count=0
    while true ; do
        time "$root/tools/search.py" \
            configuration \
            --prog "$root/src/$shader.c" \
            --search ga \
            --save $pareto/${shader}_ga_${seed}_persp \
            --seed $seed
        if [ $? = 0 ] ; then
            break
        fi
        failure_count=`expr $failure_count + 1`
        if [ $failure_count -gt 5 ] ; then
            echo "too many GA failures ($failure_count): aborting"
            exit 2
        fi
        sleep 5
    done

    echo ===== hill climb: $shader $seed =====
    failure_count=0
    while true ; do
        "$root/tools/search.py" configuration \
            --prog "$root/src/$shader.c" \
            --search hill \
            --restarts 1 \
            --initial `cat $pareto/${shader}_ga_${seed}_persp/*.genome | tr "\n" "," | sed -e 's/,$//'` \
            --save $pareto/${shader}_hillclimb_${seed}_persp
        if [ $? = 0 ] ; then
            break
        fi
        failure_count=`expr $failure_count + 1`
        if [ $failure_count -gt 5 ] ; then
            echo "too many hill climbing failures ($failure_count): aborting"
            exit 2
        fi
        sleep 5
    done
done

