#!/usr/bin/python2

import csv
from optparse import OptionParser
import os
from subprocess import call, check_call, Popen, PIPE, STDOUT
import sys

parser = OptionParser( usage = "%prog [options] -- shader-file render-flags..." )
parser.add_option(
    "--max-failures", metavar = "N", type = int, default = 5,
    help = "maximum number of failures on a seed before stopping search"
)
parser.add_option(
    "-o", "--outfile", metavar = "file",
    help = "filename to write results to"
)
parser.add_option(
    "--seeds", metavar = "N", type = int, default = 1,
    help = "number of random seeds to try"
)
parser.add_option(
    "--start", metavar = "N", type = int, default = 0,
    help = "number of the first seed to try"
)
options, args = parser.parse_args()

if len( args ) < 1:
    parser.print_help()
    exit()

options.seeds += options.start

def flush( fh = sys.stdout ):
    fh.flush()
    try:
        os.fsync( fh.fileno() )
    except:
        pass

root = os.path.dirname( os.path.abspath( sys.argv[ 0 ] ) )
renderer = os.path.join( root, "renderer" )

if options.outfile is not None:
    fh = open( options.outfile, 'w' )
else:
    fh = sys.stdout
try:
    writer = csv.writer( fh )
    need_header = True
    best = None
    for seed in range( options.start, options.seeds ):
        print >>sys.stderr, "Seed", seed
        flush( sys.stderr )

        nfail = 0
        while nfail < options.max_failures:
            p = Popen(
                [ renderer ] + args + [ "--seed", str( seed ) ],
                stdout = PIPE
            )
            lines = p.communicate()[ 0 ]
            if p.returncode != 0:
                nfail += 1
                continue
            row = lines.splitlines()[ 0 ].split( "," )
            if need_header:
                header = [ "seed", "diff", "iters" ]
                header += map( str, range( len( row ) - 2 ) )
                writer.writerow( header )
                need_header = False
            writer.writerow( [ str( seed ) ] + row )
            flush( fh )
            if best is None or best[ 0 ] < float( row[ 0 ] ):
                best = row
                best[ 0 ] = float( best[ 0 ] )
            break
        else:
            print >>sys.stderr, options.max_failures, "failures! Aborting."
            exit( 2 )
finally:
    if options.outfile is not None:
        fh.close()
    if best is not None:
        print "best coefficients:", ",".join( best[ 2: ] )

