#!/usr/bin/python2

from glob import glob
from optparse import OptionParser
import os
import shutil
from subprocess import check_call, PIPE, Popen, CalledProcessError
import sys

import utils

parser = OptionParser( usage = "%prog [options] shader target source... " )
parser.add_option(
    "--log", metavar = "file",
    help = "search log file to get genome and fitnesss from"
)
options, args = parser.parse_args()

if len( args ) < 2:
    parser.print_help()
    exit()

root = os.path.dirname( os.path.abspath( sys.argv[ 0 ] ) )
transform = os.path.join( root, "transform" )

source  = args[ 0 ]
target  = args[ 1 ]
paretos = args[ 2: ]

shader = os.path.splitext( os.path.basename( source ) )[ 0 ]
utils.check_executable( [ transform, "--help" ] )

if os.path.exists( target ):
    check_call( [ "rm", "-rf", target ] )
os.makedirs( target )

ppfile = os.path.join( target, "000000.c" )
trfile = os.path.join( target, "000001.c" )

check_call( [ "gcc", "-E", "-DFILTER_MODE_GAUSS", source, "-o", ppfile ] )
with open( "/dev/null", 'w' ) as fh:
    cmd = [ transform, ppfile, shader, "--get-statements" ]
    p = Popen( cmd, stdout = PIPE, stderr = fh )
    lines = p.communicate()[ 0 ]
    if p.returncode != 0:
        raise CalledProcessError( cmd, p.returncode )

    stmts = map( str.strip, lines.splitlines() )

    check_call( [
        transform, ppfile, shader,
            "--change-indicator", "transformed",
            "--transform", ",".join( stmts ),
            "-o", trfile
    ], stderr = fh )

if options.log is not None:
    with open( options.log ) as fh:
        for line in fh:
            line = line.split()
            if line[ 0 ] != ( "0" * len( line[ 0 ] ) ) and \
                    line[ 0 ] != ( "1" * len( line[ 0 ] ) ):
                continue
            genome = line[ 0 ]
            fitness = line[ 2 ]
            base = "00000" + genome[ 0 ]
            with open( os.path.join( target, base + ".genome" ), 'w' ) as out:
                print >>out, genome
            with open( os.path.join( target, base + ".fitness" ), 'w' ) as out:
                print >>out, fitness
else:
    for i in "01":
        with open( os.path.join( target, "00000%s.genome" % i ), 'w' ) as out:
            print >>out, i
        with open( os.path.join( target, "00000%s.fitness" % i ), 'w' ) as out:
            print >>out, i

n = 2
mapping = dict()
for pareto in paretos:
    for fname in sorted( glob( os.path.join( pareto, "*" ) ) ):
        base, ext = os.path.splitext( fname )
        if not base in mapping:
            mapping[ base ] = n
            n += 1
        newname = os.path.join( target, "%06d" % mapping[ base ] ) + ext
        shutil.copyfile( fname, newname )

